@@ -64,6 +64,16 @@ func (b *KubeAPIServerBuilder) Build(c *fi.ModelBuilderContext) error {
 		kubeAPIServer = *b.NodeupConfig.APIServerConfig.KubeAPIServer
 	}
 
+	if b.CloudProvider == kops.CloudProviderHetzner {
+		localIP, err := b.GetMetadataLocalIP()
+		if err != nil {
+			return err
+		}
+		if localIP != "" {
+			kubeAPIServer.AdvertiseAddress = localIP
+		}
+	}
+
 	if err := b.writeAuthenticationConfig(c, &kubeAPIServer); err != nil {
 		return err
 	}
@@ -378,7 +388,16 @@ func (b *KubeAPIServerBuilder) writeServerCertificate(c *fi.ModelBuilderContext,
 		// We also want to be able to reference it locally via https://127.0.0.1
 		alternateNames = append(alternateNames, "127.0.0.1")
 
-		if b.Cluster.Spec.CloudProvider == "openstack" {
+		if b.CloudProvider == kops.CloudProviderHetzner {
+			localIP, err := b.GetMetadataLocalIP()
+			if err != nil {
+				return err
+			}
+			if localIP != "" {
+				alternateNames = append(alternateNames, localIP)
+			}
+		}
+		if b.CloudProvider == kops.CloudProviderOpenstack {
 			if b.Cluster.Spec.Topology != nil && b.Cluster.Spec.Topology.Masters == kops.TopologyPrivate {
 				instanceAddress, err := getInstanceAddress()
 				if err != nil {
@@ -443,28 +462,6 @@ func (b *KubeAPIServerBuilder) writeKubeletAPICertificate(c *fi.ModelBuilderCont
 func (b *KubeAPIServerBuilder) writeStaticCredentials(c *fi.ModelBuilderContext, kubeAPIServer *kops.KubeAPIServerConfig) error {
 	pathSrvKAPI := filepath.Join(b.PathSrvKubernetes(), "kube-apiserver")
 
-	// Support for basic auth was deprecated 1.16 and removed in 1.19
-	// https://github.com/kubernetes/kubernetes/pull/89069
-	if b.IsKubernetesLT("1.19") && b.SecretStore != nil {
-		key := "kube"
-		token, err := b.SecretStore.FindSecret(key)
-		if err != nil {
-			return err
-		}
-		if token == nil {
-			return fmt.Errorf("token not found: %q", key)
-		}
-		csv := string(token.Data) + "," + adminUser + "," + adminUser + "," + adminGroup
-
-		t := &nodetasks.File{
-			Path:     filepath.Join(pathSrvKAPI, "basic_auth.csv"),
-			Contents: fi.NewStringResource(csv),
-			Type:     nodetasks.FileType_File,
-			Mode:     s("0600"),
-		}
-		c.AddTask(t)
-	}
-
 	if b.SecretStore != nil {
 		allTokens, err := b.allAuthTokens()
 		if err != nil {
@@ -489,14 +486,6 @@ func (b *KubeAPIServerBuilder) writeStaticCredentials(c *fi.ModelBuilderContext,
 		})
 	}
 
-	// Support for basic auth was deprecated 1.16 and removed in 1.19
-	// https://github.com/kubernetes/kubernetes/pull/89069
-	if b.IsKubernetesLT("1.19") {
-		if kubeAPIServer.DisableBasicAuth != nil && !*kubeAPIServer.DisableBasicAuth {
-			kubeAPIServer.BasicAuthFile = filepath.Join(pathSrvKAPI, "basic_auth.csv")
-		}
-	}
-
 	return nil
 }
 
@@ -585,10 +574,12 @@ func (b *KubeAPIServerBuilder) buildPod(kubeAPIServer *kops.KubeAPIServerConfig)
 		Path: "/healthz",
 		Port: intstr.FromInt(wellknownports.KubeAPIServerHealthCheck),
 	}
+
+	insecurePort := fi.Int32Value(kubeAPIServer.InsecurePort)
 	if useHealthcheckProxy {
 		// kube-apiserver-healthcheck sidecar container runs on port 3990
-	} else if kubeAPIServer.InsecurePort != 0 {
-		probeAction.Port = intstr.FromInt(int(kubeAPIServer.InsecurePort))
+	} else if insecurePort != 0 {
+		probeAction.Port = intstr.FromInt(int(insecurePort))
 	} else if kubeAPIServer.SecurePort != 0 {
 		probeAction.Port = intstr.FromInt(int(kubeAPIServer.SecurePort))
 		probeAction.Scheme = v1.URISchemeHTTPS
@@ -616,6 +607,9 @@ func (b *KubeAPIServerBuilder) buildPod(kubeAPIServer *kops.KubeAPIServerConfig)
 	}
 
 	image := kubeAPIServer.Image
+	if components.IsBaseURL(b.Cluster.Spec.KubernetesVersion) {
+		image = strings.Replace(image, "registry.k8s.io", "k8s.gcr.io", 1)
+	}
 	if b.Architecture != architectures.ArchitectureAmd64 {
 		image = strings.Replace(image, "-amd64", "-"+string(b.Architecture), 1)
 	}
@@ -644,16 +638,16 @@ func (b *KubeAPIServerBuilder) buildPod(kubeAPIServer *kops.KubeAPIServerConfig)
 		},
 	}
 
-	if kubeAPIServer.InsecurePort != 0 {
+	if insecurePort != 0 {
 		container.Ports = append(container.Ports, v1.ContainerPort{
 			Name:          "local",
-			ContainerPort: kubeAPIServer.InsecurePort,
-			HostPort:      kubeAPIServer.InsecurePort,
+			ContainerPort: insecurePort,
+			HostPort:      insecurePort,
 		})
 	}
 
 	// Log both to docker and to the logfile
-	addHostPathMapping(pod, container, "logfile", "/var/log/kube-apiserver.log").ReadOnly = false
+	kubemanifest.AddHostPathMapping(pod, container, "logfile", "/var/log/kube-apiserver.log").WithReadWrite()
 	// We use lighter containers that don't include shells
 	// But they have richer logging support via klog
 	if b.IsKubernetesGTE("1.23") {
@@ -681,22 +675,22 @@ func (b *KubeAPIServerBuilder) buildPod(kubeAPIServer *kops.KubeAPIServerConfig)
 
 	for _, path := range b.SSLHostPaths() {
 		name := strings.Replace(path, "/", "", -1)
-		addHostPathMapping(pod, container, name, path)
+		kubemanifest.AddHostPathMapping(pod, container, name, path)
 	}
 
 	// Add cloud config file if needed
 	if b.Cluster.Spec.CloudConfig != nil {
-		addHostPathMapping(pod, container, "cloudconfig", InTreeCloudConfigFilePath)
+		kubemanifest.AddHostPathMapping(pod, container, "cloudconfig", InTreeCloudConfigFilePath)
 	}
 
-	addHostPathMapping(pod, container, "kubernetesca", filepath.Join(b.PathSrvKubernetes(), "ca.crt"))
+	kubemanifest.AddHostPathMapping(pod, container, "kubernetesca", filepath.Join(b.PathSrvKubernetes(), "ca.crt"))
 
 	pathSrvKAPI := filepath.Join(b.PathSrvKubernetes(), "kube-apiserver")
-	addHostPathMapping(pod, container, "srvkapi", pathSrvKAPI)
+	kubemanifest.AddHostPathMapping(pod, container, "srvkapi", pathSrvKAPI)
 
 	pathSrvSshproxy := b.PathSrvSshproxy()
 	if pathSrvSshproxy != "" {
-		addHostPathMapping(pod, container, "srvsshproxy", pathSrvSshproxy)
+		kubemanifest.AddHostPathMapping(pod, container, "srvsshproxy", pathSrvSshproxy)
 	}
 
 	auditLogPath := kubeAPIServer.AuditLogPath
@@ -707,12 +701,12 @@ func (b *KubeAPIServerBuilder) buildPod(kubeAPIServer *kops.KubeAPIServerConfig)
 		// Renaming is not possible when the file is mounted as the host path, and will return a
 		// 'Device or resource busy' error
 		auditLogPathDir := filepath.Dir(*auditLogPath)
-		addHostPathMapping(pod, container, "auditlogpathdir", auditLogPathDir).ReadOnly = false
+		kubemanifest.AddHostPathMapping(pod, container, "auditlogpathdir", auditLogPathDir).WithReadWrite()
 	}
 
 	if b.Cluster.Spec.Authentication != nil {
 		if b.Cluster.Spec.Authentication.Kopeio != nil || b.Cluster.Spec.Authentication.AWS != nil {
-			addHostPathMapping(pod, container, "authn-config", PathAuthnConfig)
+			kubemanifest.AddHostPathMapping(pod, container, "authn-config", PathAuthnConfig)
 		}
 	}
 
