@@ -16,6 +16,9 @@ limitations under the License.
 
 package kops
 
+// NvidiaDefaultDriverPackage is the nvidia driver default version
+const NvidiaDefaultDriverPackage = "nvidia-headless-510-server"
+
 // ContainerdConfig is the configuration for containerd
 type ContainerdConfig struct {
 	// Address of containerd's GRPC server (default "/run/containerd/containerd.sock").
@@ -38,13 +41,22 @@ type ContainerdConfig struct {
 	Version *string `json:"version,omitempty"`
 	// NvidiaGPU configures the Nvidia GPU runtime.
 	NvidiaGPU *NvidiaGPUConfig `json:"nvidiaGPU,omitempty"`
+	// Runc configures the runc runtime.
+	Runc *Runc `json:"runc,omitempty"`
 }
 
 type NvidiaGPUConfig struct {
 	// Package is the name of the nvidia driver package that will be installed.
-	// Default is "nvidia-headless-460-server".
+	// Default is "nvidia-headless-510-server".
 	DriverPackage string `json:"package,omitempty"`
 	// Enabled determines if kOps will install the Nvidia GPU runtime and drivers.
 	// They will only be installed on intances that has an Nvidia GPU.
 	Enabled *bool `json:"enabled,omitempty"`
 }
+
+type Runc struct {
+	// Version used to pick the runc package.
+	Version *string `json:"version,omitempty"`
+	// Packages overrides the URL and hash for the packages.
+	Packages *PackagesConfig `json:"packages,omitempty"`
+}
