@@ -22,9 +22,9 @@ import (
 
 // UseKopsControllerForNodeBootstrap is true if nodeup should use kops-controller for bootstrapping.
 func UseKopsControllerForNodeBootstrap(cluster *kops.Cluster) bool {
-	switch kops.CloudProviderID(cluster.Spec.CloudProvider) {
+	switch cluster.Spec.GetCloudProvider() {
 	case kops.CloudProviderAWS:
-		return cluster.IsKubernetesGTE("1.19")
+		return true
 	case kops.CloudProviderGCE:
 		return cluster.IsKubernetesGTE("1.22")
 	default:
