@@ -73,7 +73,7 @@ type KubeletConfigSpec struct {
 	// ClusterDNS is the IP address for a cluster DNS server
 	ClusterDNS string `json:"clusterDNS,omitempty" flag:"cluster-dns"`
 	// NetworkPluginName is the name of the network plugin to be invoked for various events in kubelet/pod lifecycle
-	NetworkPluginName string `json:"networkPluginName,omitempty" flag:"network-plugin"`
+	NetworkPluginName *string `json:"networkPluginName,omitempty" flag:"network-plugin"`
 	// CloudProvider is the provider for cloud services.
 	CloudProvider string `json:"cloudProvider,omitempty" flag:"cloud-provider"`
 	// KubeletCgroups is the absolute name of cgroups to isolate the kubelet in.
@@ -124,7 +124,7 @@ type KubeletConfigSpec struct {
 	// NodeLabels to add when registering the node in the cluster.
 	NodeLabels map[string]string `json:"nodeLabels,omitempty" flag:"node-labels"`
 	// NonMasqueradeCIDR configures masquerading: traffic to IPs outside this range will use IP masquerade.
-	NonMasqueradeCIDR string `json:"nonMasqueradeCIDR,omitempty" flag:"non-masquerade-cidr"`
+	NonMasqueradeCIDR *string `json:"nonMasqueradeCIDR,omitempty" flag:"non-masquerade-cidr"`
 	// Enable gathering custom metrics.
 	EnableCustomMetrics *bool `json:"enableCustomMetrics,omitempty" flag:"enable-custom-metrics"`
 	// NetworkPluginMTU is the MTU to be passed to the network plugin,
@@ -294,9 +294,11 @@ type KubeAPIServerConfig struct {
 	// SecurePort is the port the kube runs on
 	SecurePort int32 `json:"securePort,omitempty" flag:"secure-port"`
 	// InsecurePort is the port the insecure api runs
-	InsecurePort int32 `json:"insecurePort,omitempty" flag:"insecure-port"`
+	InsecurePort *int32 `json:"insecurePort,omitempty" flag:"insecure-port"`
 	// Address is the binding address for the kube api: Deprecated - use insecure-bind-address and bind-address
 	Address string `json:"address,omitempty" flag:"address"`
+	// AdvertiseAddress is the IP address on which to advertise the apiserver to members of the cluster.
+	AdvertiseAddress string `json:"advertiseAddress,omitempty" flag:"advertise-address"`
 	// BindAddress is the binding address for the secure kubernetes API
 	BindAddress string `json:"bindAddress,omitempty" flag:"bind-address"`
 	// InsecureBindAddress is the binding address for the InsecurePort for the insecure kubernetes API
@@ -678,6 +680,8 @@ type CloudControllerManagerConfig struct {
 	AllocateNodeCIDRs *bool `json:"allocateNodeCIDRs,omitempty" flag:"allocate-node-cidrs"`
 	// ConfigureCloudRoutes enables CIDRs allocated with to be configured on the cloud provider.
 	ConfigureCloudRoutes *bool `json:"configureCloudRoutes,omitempty" flag:"configure-cloud-routes"`
+	// Controllers is a list of controllers to enable on the controller-manager
+	Controllers []string `json:"controllers,omitempty" flag:"controllers"`
 	// CIDRAllocatorType specifies the type of CIDR allocator to use.
 	CIDRAllocatorType *string `json:"cidrAllocatorType,omitempty" flag:"cidr-allocator-type"`
 	// LeaderElection defines the configuration of leader election client.
@@ -686,6 +690,9 @@ type CloudControllerManagerConfig struct {
 	UseServiceAccountCredentials *bool `json:"useServiceAccountCredentials,omitempty" flag:"use-service-account-credentials"`
 	// EnableLeaderMigration enables controller leader migration.
 	EnableLeaderMigration *bool `json:"enableLeaderMigration,omitempty" flag:"enable-leader-migration"`
+	// CPURequest of CloudControllerManager container.
+	// Default: 200m
+	CPURequest *resource.Quantity `json:"cpuRequest,omitempty"`
 }
 
 // KubeSchedulerConfig is the configuration for the kube-scheduler
@@ -799,6 +806,9 @@ type OpenstackRouter struct {
 // OpenstackNetwork defines the config for a network
 type OpenstackNetwork struct {
 	AvailabilityZoneHints []*string `json:"availabilityZoneHints,omitempty"`
+	IPv6SupportDisabled   *bool     `json:"ipv6SupportDisabled,omitempty"`
+	PublicNetworkNames    []*string `json:"publicNetworkNames,omitempty"`
+	InternalNetworkNames  []*string `json:"internalNetworkNames,omitempty"`
 }
 
 // OpenstackMetadata defines config for metadata service related settings
@@ -807,8 +817,8 @@ type OpenstackMetadata struct {
 	ConfigDrive *bool `json:"configDrive,omitempty"`
 }
 
-// OpenstackConfiguration defines cloud config elements for the openstack cloud provider
-type OpenstackConfiguration struct {
+// OpenstackSpec defines cloud config elements for the openstack cloud provider
+type OpenstackSpec struct {
 	Loadbalancer       *OpenstackLoadbalancerConfig `json:"loadbalancer,omitempty"`
 	Monitor            *OpenstackMonitor            `json:"monitor,omitempty"`
 	Router             *OpenstackRouter             `json:"router,omitempty"`
@@ -818,8 +828,8 @@ type OpenstackConfiguration struct {
 	Metadata           *OpenstackMetadata           `json:"metadata,omitempty"`
 }
 
-// AzureConfiguration defines Azure specific cluster configuration.
-type AzureConfiguration struct {
+// AzureSpec defines Azure specific cluster configuration.
+type AzureSpec struct {
 	// SubscriptionID specifies the subscription used for the cluster installation.
 	SubscriptionID string `json:"subscriptionId,omitempty"`
 	// TenantID is the ID of the tenant that the cluster is deployed in.
@@ -882,9 +892,11 @@ type CloudConfiguration struct {
 	SpotinstProduct     *string `json:"spotinstProduct,omitempty"`
 	SpotinstOrientation *string `json:"spotinstOrientation,omitempty"`
 	// Openstack cloud-config options
-	Openstack *OpenstackConfiguration `json:"openstack,omitempty"`
+	// +k8s:conversion-gen=false
+	Openstack *OpenstackSpec `json:"openstack,omitempty"`
 	// Azure cloud-config options
-	Azure *AzureConfiguration `json:"azure,omitempty"`
+	// +k8s:conversion-gen=false
+	Azure *AzureSpec `json:"azure,omitempty"`
 	// AWSEBSCSIDriver is the config for the AWS EBS CSI driver
 	AWSEBSCSIDriver *AWSEBSCSIDriver `json:"awsEBSCSIDriver,omitempty"`
 	// GCPPDCSIDriver is the config for the GCP PD CSI driver
@@ -906,6 +918,10 @@ type AWSEBSCSIDriver struct {
 	// If not specified, the value is approximated from the instance type.
 	// Default: -
 	VolumeAttachLimit *int `json:"volumeAttachLimit,omitempty"`
+
+	// PodAnnotations are the annotations added to AWS EBS CSI node and controller Pods.
+	// Default: none
+	PodAnnotations map[string]string `json:"podAnnotations,omitempty"`
 }
 
 // GCPPDCSIDriver is the config for the GCP PD CSI driver
@@ -946,6 +962,10 @@ type NodeTerminationHandlerConfig struct {
 	// EnableSQSTerminationDraining enables queue-processor mode which drains nodes when an SQS termination event is received.
 	EnableSQSTerminationDraining *bool `json:"enableSQSTerminationDraining,omitempty"`
 
+	// ExcludeFromLoadBalancers makes node termination handler will mark for exclusion from load balancers before node are cordoned.
+	// Default: true
+	ExcludeFromLoadBalancers *bool `json:"excludeFromLoadBalancers,omitempty"`
+
 	// ManagedASGTag is the tag used to determine which nodes NTH can take action on
 	ManagedASGTag *string `json:"managedASGTag,omitempty"`
 
@@ -955,6 +975,8 @@ type NodeTerminationHandlerConfig struct {
 	// CPURequest of NodeTerminationHandler container.
 	// Default: 50m
 	CPURequest *resource.Quantity `json:"cpuRequest,omitempty"`
+	// Version is the container image tag used.
+	Version *string `json:"version,omitempty"`
 }
 
 // NodeProblemDetector determines the node problem detector configuration.
@@ -1057,6 +1079,10 @@ type CertManagerConfig struct {
 	// defaultIssuer sets a default clusterIssuer
 	// Default: none
 	DefaultIssuer *string `json:"defaultIssuer,omitempty"`
+
+	// nameservers is a list of nameserver IP addresses to use instead of the pod defaults.
+	// Default: none
+	Nameservers []string `json:"nameservers,omitempty"`
 }
 
 // AWSLoadBalancerControllerConfig determines the AWS LB controller configuration.
@@ -1066,6 +1092,15 @@ type AWSLoadBalancerControllerConfig struct {
 	Enabled *bool `json:"enabled,omitempty"`
 	// Version is the container image tag used.
 	Version *string `json:"version,omitempty"`
+	// EnableWAF specifies whether the controller can use WAFs (Classic Regional).
+	// Default: false
+	EnableWAF bool `json:"enableWAF,omitempty"`
+	// EnableWAFv2 specifies whether the controller can use WAFs (V2).
+	// Default: false
+	EnableWAFv2 bool `json:"enableWAFv2,omitempty"`
+	// EnableShield specifies whether the controller can enable Shield Advanced.
+	// Default: false
+	EnableShield bool `json:"enableShield,omitempty"`
 }
 
 // HasAdmissionController checks if a specific admission controller is enabled
