@@ -25,16 +25,14 @@ func SupportedClouds() []kops.CloudProviderID {
 	clouds := []kops.CloudProviderID{
 		kops.CloudProviderAWS,
 		kops.CloudProviderDO,
+		kops.CloudProviderGCE,
 		kops.CloudProviderOpenstack,
 	}
-	if featureflag.AlphaAllowALI.Enabled() {
-		clouds = append(clouds, kops.CloudProviderALI)
-	}
 	if featureflag.Azure.Enabled() {
 		clouds = append(clouds, kops.CloudProviderAzure)
 	}
-	if featureflag.AlphaAllowGCE.Enabled() {
-		clouds = append(clouds, kops.CloudProviderGCE)
+	if featureflag.Hetzner.Enabled() {
+		clouds = append(clouds, kops.CloudProviderHetzner)
 	}
 
 	return clouds
