@@ -56,14 +56,14 @@ var (
 	ExperimentalClusterDNS = new("ExperimentalClusterDNS", Bool(false))
 	// GoogleCloudBucketACL means the ACL will be set on a bucket when using GCS
 	GoogleCloudBucketACL = new("GoogleCloudBucketAcl", Bool(false))
-	// KeepLaunchConfigurations can be set to prevent garbage collection of old launch configurations
-	KeepLaunchConfigurations = new("KeepLaunchConfigurations", Bool(false))
 	// SpecOverrideFlag allows setting spec values on create
 	SpecOverrideFlag = new("SpecOverrideFlag", Bool(false))
 	// Spotinst toggles the use of Spotinst integration.
 	Spotinst = new("Spotinst", Bool(false))
 	// SpotinstOcean toggles the use of Spotinst Ocean instance group implementation.
 	SpotinstOcean = new("SpotinstOcean", Bool(false))
+	// SpotinstOceanTemplate toggles the use of Spotinst Ocean object as a template for Virtual Node Groups.
+	SpotinstOceanTemplate = new("SpotinstOceanTemplate", Bool(false))
 	// SpotinstHybrid toggles between hybrid and full instance group implementations.
 	SpotinstHybrid = new("SpotinstHybrid", Bool(false))
 	// SpotinstController toggles the installation of the Spotinst controller addon.
@@ -84,14 +84,14 @@ var (
 	APIServerNodes = new("APIServerNodes", Bool(false))
 	// UseAddonOperators activates experimental addon operator support
 	UseAddonOperators = new("UseAddonOperators", Bool(false))
-	// AWSIPv6 activates experimental AWS IPv6 support.
-	AWSIPv6 = new("AWSIPv6", Bool(false))
 	// TerraformManagedFiles enables rendering managed files into the Terraform configuration.
 	TerraformManagedFiles = new("TerraformManagedFiles", Bool(true))
-	// AlphaAllowGCE is a feature flag that gates GCE support while it is alpha.
-	AlphaAllowGCE = new("AlphaAllowGCE", Bool(false))
-	// AlphaAllowALI is a feature flag that gates aliyun support while it is alpha.
-	AlphaAllowALI = new("AlphaAllowALI", Bool(false))
+	// Karpenter enables karpenter-managed Instance Groups
+	Karpenter = new("Karpenter", Bool(false))
+	// ImageDigest remaps all manifests with image digests
+	ImageDigest = new("ImageDigest", Bool(true))
+	// Hetzner toggles the Hetzner Cloud support.
+	Hetzner = new("Hetzner", Bool(false))
 )
 
 // FeatureFlag defines a feature flag
