@@ -34,10 +34,11 @@ import (
 	"k8s.io/apimachinery/pkg/util/json"
 	"k8s.io/apimachinery/pkg/util/strategicpatch"
 	"k8s.io/klog/v2"
+	"k8s.io/kubectl/pkg/drain"
+
 	api "k8s.io/kops/pkg/apis/kops"
 	"k8s.io/kops/pkg/cloudinstances"
 	"k8s.io/kops/pkg/validation"
-	"k8s.io/kubectl/pkg/drain"
 )
 
 const rollingUpdateTaintKey = "kops.k8s.io/scheduled-for-update"
@@ -156,6 +157,7 @@ func (c *RollingUpdateCluster) rollingUpdateInstanceGroup(group *cloudinstances.
 				if err := c.detachInstance(u); err != nil {
 					// If detaching a node fails, we simply proceed to the next one instead of
 					// bubbling up the error.
+					klog.Errorf("Failed to detach instance %q: %v", u.ID, err)
 					skippedNodes++
 					numSurge--
 					if maxSurge > len(update)-skippedNodes {
@@ -409,7 +411,7 @@ func (c *RollingUpdateCluster) drainTerminateAndWait(u *cloudinstances.CloudInst
 	}
 
 	// GCE often re-uses names, so we delete the node object to prevent the new instance from using the cordoned Node object
-	if api.CloudProviderID(c.Cluster.Spec.CloudProvider) == api.CloudProviderGCE && !isBastion && !c.CloudOnly {
+	if c.Cluster.Spec.GetCloudProvider() == api.CloudProviderGCE && !isBastion && !c.CloudOnly {
 		if u.Node == nil {
 			klog.Warningf("no kubernetes Node associated with %s, skipping node deletion", instanceID)
 		} else {
@@ -438,8 +440,9 @@ func (c *RollingUpdateCluster) drainTerminateAndWait(u *cloudinstances.CloudInst
 }
 
 func (c *RollingUpdateCluster) reconcileInstanceGroup() error {
-	if api.CloudProviderID(c.Cluster.Spec.CloudProvider) != api.CloudProviderOpenstack &&
-		api.CloudProviderID(c.Cluster.Spec.CloudProvider) != api.CloudProviderDO {
+	if c.Cluster.Spec.GetCloudProvider() != api.CloudProviderOpenstack &&
+		c.Cluster.Spec.GetCloudProvider() != api.CloudProviderHetzner &&
+		c.Cluster.Spec.GetCloudProvider() != api.CloudProviderDO {
 		return nil
 	}
 	rto := fi.RunTasksOptions{}
@@ -642,6 +645,10 @@ func (c *RollingUpdateCluster) drainNode(u *cloudinstances.CloudInstance) error
 		return fmt.Errorf("error excluding node from load balancer: %v", err)
 	}
 
+	if err := c.Cloud.DeregisterInstance(u); err != nil {
+		return fmt.Errorf("error deregistering instance %q, node %q: %v", u.ID, u.Node.Name, err)
+	}
+
 	if err := drain.RunNodeDrain(helper, u.Node.Name); err != nil {
 		if apierrors.IsNotFound(err) {
 			return nil
@@ -677,7 +684,7 @@ func (c *RollingUpdateCluster) UpdateSingleInstance(cloudMember *cloudinstances.
 	if detach {
 		if cloudMember.CloudInstanceGroup.InstanceGroup.IsMaster() {
 			klog.Warning("cannot detach master instances. Assuming --surge=false")
-		} else {
+		} else if cloudMember.CloudInstanceGroup.InstanceGroup.Spec.Manager != api.InstanceManagerKarpenter {
 			err := c.detachInstance(cloudMember)
 			if err != nil {
 				return fmt.Errorf("failed to detach instance: %v", err)
