@@ -22,6 +22,7 @@ import (
 	"strings"
 
 	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
+	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/klog/v2"
 	"k8s.io/kops/util/pkg/text"
 	"sigs.k8s.io/yaml"
@@ -42,6 +43,15 @@ func (o *Object) ToUnstructured() *unstructured.Unstructured {
 	return &unstructured.Unstructured{Object: o.data}
 }
 
+// FromRuntimeObject converts from a runtime.Object.
+func FromRuntimeObject(obj runtime.Object) (*Object, error) {
+	data, err := runtime.DefaultUnstructuredConverter.ToUnstructured(obj)
+	if err != nil {
+		return nil, fmt.Errorf("unable to convert %T to unstructured: %w", obj, err)
+	}
+	return NewObject(data), nil
+}
+
 // ObjectList describes a list of objects, allowing us to add bulk-methods
 type ObjectList []*Object
 
@@ -60,6 +70,7 @@ func LoadObjectsFrom(contents []byte) (ObjectList, error) {
 		data := make(map[string]interface{})
 		err := yaml.Unmarshal(section, &data)
 		if err != nil {
+			klog.Infof("invalid YAML section: %s", string(section))
 			return nil, fmt.Errorf("error parsing yaml: %v", err)
 		}
 
