@@ -101,7 +101,7 @@ func (b *APILoadBalancerBuilder) Build(c *fi.ModelBuilderContext) error {
 					continue
 				}
 
-			case kops.SubnetTypePrivate:
+			case kops.SubnetTypeDualStack, kops.SubnetTypePrivate:
 				if lbSpec.Type != kops.LoadBalancerTypeInternal {
 					continue
 				}
@@ -184,9 +184,7 @@ func (b *APILoadBalancerBuilder) Build(c *fi.ModelBuilderContext) error {
 			Type:          fi.String("network"),
 			IpAddressType: fi.String("ipv4"),
 		}
-		// DualStack can only be used for public NLB
-		// https://aws.amazon.com/premiumsupport/knowledge-center/elb-configure-with-ipv6
-		if b.UseIPv6ForAPI() && lbSpec.Type == kops.LoadBalancerTypePublic {
+		if b.UseIPv6ForAPI() {
 			nlb.IpAddressType = fi.String("dualstack")
 		}
 
@@ -552,7 +550,7 @@ func (a ByScoreDescending) Less(i, j int) bool {
 
 // Choose between subnets in a zone.
 // We have already applied the rules to match internal subnets to internal ELBs and vice-versa for public-facing ELBs.
-// For internal ELBs: we prefer the master subnets
+// For internal ELBs: we prefer dual stack and the master subnets
 // For public facing ELBs: we prefer the utility subnets
 func (b *APILoadBalancerBuilder) chooseBestSubnetForELB(zone string, subnets []*kops.ClusterSubnetSpec) *kops.ClusterSubnetSpec {
 	if len(subnets) == 0 {
@@ -577,8 +575,12 @@ func (b *APILoadBalancerBuilder) chooseBestSubnetForELB(zone string, subnets []*
 			score += 1
 		}
 
+		if subnet.Type == kops.SubnetTypeDualStack {
+			score += 2
+		}
+
 		if subnet.Type == kops.SubnetTypeUtility {
-			score += 1
+			score += 3
 		}
 
 		scoredSubnets = append(scoredSubnets, &scoredSubnet{
