@@ -23,7 +23,9 @@ import (
 
 	"github.com/aws/aws-sdk-go/aws"
 	"github.com/aws/aws-sdk-go/service/ec2"
+	"k8s.io/apimachinery/pkg/api/resource"
 	"k8s.io/klog/v2"
+
 	"k8s.io/kops/pkg/apis/kops"
 	"k8s.io/kops/pkg/featureflag"
 	"k8s.io/kops/pkg/model"
@@ -79,12 +81,14 @@ func (b *AutoscalingGroupModelBuilder) Build(c *fi.ModelBuilderContext) error {
 		c.AddTask(task)
 
 		// @step: now lets build the autoscaling group task
-		tsk, err := b.buildAutoScalingGroupTask(c, name, ig)
-		if err != nil {
-			return err
+		if ig.Spec.Manager != "Karpenter" {
+			tsk, err := b.buildAutoScalingGroupTask(c, name, ig)
+			if err != nil {
+				return err
+			}
+			tsk.LaunchTemplate = task
+			c.AddTask(tsk)
 		}
-		tsk.LaunchTemplate = task
-		c.AddTask(tsk)
 
 		warmPool := b.Cluster.Spec.WarmPool.ResolveDefaults(ig)
 		{
@@ -184,7 +188,6 @@ func (b *AutoscalingGroupModelBuilder) buildLaunchTemplateTask(c *fi.ModelBuilde
 		ImageID:                      fi.String(ig.Spec.Image),
 		InstanceInterruptionBehavior: ig.Spec.InstanceInterruptionBehavior,
 		InstanceMonitoring:           fi.Bool(false),
-		InstanceType:                 fi.String(strings.Split(ig.Spec.MachineType, ",")[0]),
 		IPv6AddressCount:             fi.Int64(0),
 		RootVolumeIops:               fi.Int64(int64(fi.Int32Value(ig.Spec.RootVolumeIOPS))),
 		RootVolumeOptimization:       ig.Spec.RootVolumeOptimization,
@@ -197,6 +200,10 @@ func (b *AutoscalingGroupModelBuilder) buildLaunchTemplateTask(c *fi.ModelBuilde
 		UserData:                     userData,
 	}
 
+	if ig.Spec.Manager == kops.InstanceManagerCloudGroup {
+		lt.InstanceType = fi.String(strings.Split(ig.Spec.MachineType, ",")[0])
+	}
+
 	{
 		// @step: check the subnets are ok and pull together an array for us
 		subnets, err := b.GatherSubnets(ig)
@@ -204,14 +211,14 @@ func (b *AutoscalingGroupModelBuilder) buildLaunchTemplateTask(c *fi.ModelBuilde
 			return nil, err
 		}
 
-		// @step: check if we can add an public ip to this subnet
+		// @step: check if we can add a public ip to this subnet
 		switch subnets[0].Type {
 		case kops.SubnetTypePublic, kops.SubnetTypeUtility:
 			lt.AssociatePublicIP = fi.Bool(true)
 			if ig.Spec.AssociatePublicIP != nil {
 				lt.AssociatePublicIP = ig.Spec.AssociatePublicIP
 			}
-		case kops.SubnetTypePrivate:
+		case kops.SubnetTypeDualStack, kops.SubnetTypePrivate:
 			lt.AssociatePublicIP = fi.Bool(false)
 		}
 
@@ -489,17 +496,62 @@ func (b *AutoscalingGroupModelBuilder) buildAutoScalingGroupTask(c *fi.ModelBuil
 	sort.Stable(awstasks.OrderTargetGroupsByName(t.TargetGroups))
 
 	// @step: are we using a mixed instance policy
-	if ig.Spec.MixedInstancesPolicy != nil {
+	if ig.Spec.MixedInstancesPolicy != nil && ig.Spec.Manager == kops.InstanceManagerCloudGroup {
 		spec := ig.Spec.MixedInstancesPolicy
 
+		if spec.InstanceRequirements != nil {
+
+			ir := &awstasks.InstanceRequirements{}
+
+			cpu := spec.InstanceRequirements.CPU
+			if cpu != nil {
+				if cpu.Max != nil {
+					cpuMax, _ := spec.InstanceRequirements.CPU.Max.AsInt64()
+					ir.CPUMax = &cpuMax
+				}
+				if cpu.Min != nil {
+					cpuMin, _ := spec.InstanceRequirements.CPU.Min.AsInt64()
+					ir.CPUMin = &cpuMin
+				}
+			} else {
+				ir.CPUMin = fi.Int64(0)
+			}
+
+			memory := spec.InstanceRequirements.Memory
+			if memory != nil {
+				if memory.Max != nil {
+					memoryMax := spec.InstanceRequirements.Memory.Max.ScaledValue(resource.Mega)
+					ir.MemoryMax = &memoryMax
+				}
+				if memory.Min != nil {
+					memoryMin := spec.InstanceRequirements.Memory.Min.ScaledValue(resource.Mega)
+					ir.MemoryMin = &memoryMin
+				}
+			} else {
+				ir.MemoryMin = fi.Int64(0)
+			}
+			t.InstanceRequirements = ir
+		}
+
 		t.MixedInstanceOverrides = spec.Instances
 		t.MixedOnDemandAboveBase = spec.OnDemandAboveBase
 		t.MixedOnDemandAllocationStrategy = spec.OnDemandAllocationStrategy
 		t.MixedOnDemandBase = spec.OnDemandBase
 		t.MixedSpotAllocationStrategy = spec.SpotAllocationStrategy
 		t.MixedSpotInstancePools = spec.SpotInstancePools
-		t.MixedSpotMaxPrice = ig.Spec.MaxPrice
+		// In order to unset maxprice, the value needs to be ""
+		if ig.Spec.MaxPrice == nil {
+			t.MixedSpotMaxPrice = fi.String("")
+		} else {
+			t.MixedSpotMaxPrice = ig.Spec.MaxPrice
+		}
 	}
 
+	if ig.Spec.MaxInstanceLifetime != nil {
+		lifetimeSec := int64(ig.Spec.MaxInstanceLifetime.Seconds())
+		t.MaxInstanceLifetime = fi.Int64(lifetimeSec)
+	} else {
+		t.MaxInstanceLifetime = fi.Int64(0)
+	}
 	return t, nil
 }
