@@ -25,6 +25,7 @@ import (
 	"k8s.io/kops/upup/pkg/fi/cloudup/awsup"
 
 	"github.com/aws/aws-sdk-go/aws"
+
 	"k8s.io/kops/pkg/apis/kops"
 	"k8s.io/kops/upup/pkg/fi"
 )
@@ -69,6 +70,10 @@ var (
 			name:    "InstanceStateChange",
 			pattern: `{"source": ["aws.ec2"],"detail-type": ["EC2 Instance State-change Notification"]}`,
 		},
+		{
+			name:    "InstanceScheduledChange",
+			pattern: `{"source": ["aws.health"],"detail-type": ["AWS Health Event"],"detail": {"service": ["EC2"],"eventTypeCategory": ["scheduledChange"]}}`,
+		},
 	}
 )
 
@@ -80,9 +85,11 @@ type NodeTerminationHandlerBuilder struct {
 
 func (b *NodeTerminationHandlerBuilder) Build(c *fi.ModelBuilderContext) error {
 	for _, ig := range b.InstanceGroups {
-		err := b.configureASG(c, ig)
-		if err != nil {
-			return err
+		if ig.Spec.Manager == kops.InstanceManagerCloudGroup {
+			err := b.configureASG(c, ig)
+			if err != nil {
+				return err
+			}
 		}
 	}
 
