@@ -18,7 +18,6 @@ package model
 
 import (
 	"bytes"
-	"compress/gzip"
 	"crypto/sha256"
 	"encoding/base64"
 	"fmt"
@@ -26,7 +25,6 @@ import (
 	"sort"
 	"strconv"
 	"strings"
-	"text/template"
 
 	"k8s.io/klog/v2"
 	"k8s.io/kops/pkg/apis/kops/model"
@@ -44,7 +42,7 @@ import (
 )
 
 type NodeUpConfigBuilder interface {
-	BuildConfig(ig *kops.InstanceGroup, apiserverAdditionalIPs []string, caTasks map[string]*fitasks.Keypair) (*nodeup.Config, *nodeup.BootConfig, error)
+	BuildConfig(ig *kops.InstanceGroup, apiserverAdditionalIPs []string, keysets map[string]*fi.Keyset) (*nodeup.Config, *nodeup.BootConfig, error)
 }
 
 // BootstrapScriptBuilder creates the bootstrap script
@@ -83,20 +81,33 @@ func (b *BootstrapScript) kubeEnv(ig *kops.InstanceGroup, c *fi.Context) (string
 	var alternateNames []string
 
 	for _, hasAddress := range b.alternateNameTasks {
-		address, err := hasAddress.FindIPAddress(c)
+		addresses, err := hasAddress.FindAddresses(c)
 		if err != nil {
 			return "", fmt.Errorf("error finding address for %v: %v", hasAddress, err)
 		}
-		if address == nil {
-			klog.Warningf("Task did not have an address: %v", hasAddress)
+		if len(addresses) == 0 {
+			// Such tasks won't have an address in dry-run mode, until the resource is created
+			klog.V(2).Infof("Task did not have an address: %v", hasAddress)
 			continue
 		}
-		klog.V(8).Infof("Resolved alternateName %q for %q", *address, hasAddress)
-		alternateNames = append(alternateNames, *address)
+		for _, address := range addresses {
+			klog.V(8).Infof("Resolved alternateName %q for %q", address, hasAddress)
+			alternateNames = append(alternateNames, address)
+		}
 	}
 
 	sort.Strings(alternateNames)
-	config, bootConfig, err := b.builder.NodeUpConfigBuilder.BuildConfig(ig, alternateNames, b.caTasks)
+
+	keysets := make(map[string]*fi.Keyset)
+	for _, caTask := range b.caTasks {
+		name := *caTask.Name
+		keyset := caTask.Keyset()
+		if keyset == nil {
+			return "", fmt.Errorf("failed to get keyset from %q", name)
+		}
+		keysets[name] = keyset
+	}
+	config, bootConfig, err := b.builder.NodeUpConfigBuilder.BuildConfig(ig, alternateNames, keysets)
 	if err != nil {
 		return "", err
 	}
@@ -131,7 +142,7 @@ func (b *BootstrapScript) buildEnvironmentVariables(cluster *kops.Cluster) (map[
 		env["S3_SECRET_ACCESS_KEY"] = os.Getenv("S3_SECRET_ACCESS_KEY")
 	}
 
-	if kops.CloudProviderID(cluster.Spec.CloudProvider) == kops.CloudProviderOpenstack {
+	if cluster.Spec.GetCloudProvider() == kops.CloudProviderOpenstack {
 
 		osEnvs := []string{
 			"OS_TENANT_ID", "OS_TENANT_NAME", "OS_PROJECT_ID", "OS_PROJECT_NAME",
@@ -168,14 +179,21 @@ func (b *BootstrapScript) buildEnvironmentVariables(cluster *kops.Cluster) (map[
 		}
 	}
 
-	if kops.CloudProviderID(cluster.Spec.CloudProvider) == kops.CloudProviderDO {
+	if cluster.Spec.GetCloudProvider() == kops.CloudProviderDO {
 		doToken := os.Getenv("DIGITALOCEAN_ACCESS_TOKEN")
 		if doToken != "" {
 			env["DIGITALOCEAN_ACCESS_TOKEN"] = doToken
 		}
 	}
 
-	if kops.CloudProviderID(cluster.Spec.CloudProvider) == kops.CloudProviderAWS {
+	if cluster.Spec.GetCloudProvider() == kops.CloudProviderHetzner {
+		hcloudToken := os.Getenv("HCLOUD_TOKEN")
+		if hcloudToken != "" {
+			env["HCLOUD_TOKEN"] = hcloudToken
+		}
+	}
+
+	if cluster.Spec.GetCloudProvider() == kops.CloudProviderAWS {
 		region, err := awsup.FindRegion(cluster)
 		if err != nil {
 			return nil, err
@@ -187,24 +205,7 @@ func (b *BootstrapScript) buildEnvironmentVariables(cluster *kops.Cluster) (map[
 		}
 	}
 
-	if kops.CloudProviderID(cluster.Spec.CloudProvider) == kops.CloudProviderALI {
-		region := os.Getenv("OSS_REGION")
-		if region != "" {
-			env["OSS_REGION"] = os.Getenv("OSS_REGION")
-		}
-
-		aliID := os.Getenv("ALIYUN_ACCESS_KEY_ID")
-		if aliID != "" {
-			env["ALIYUN_ACCESS_KEY_ID"] = os.Getenv("ALIYUN_ACCESS_KEY_ID")
-		}
-
-		aliSecret := os.Getenv("ALIYUN_ACCESS_KEY_SECRET")
-		if aliSecret != "" {
-			env["ALIYUN_ACCESS_KEY_SECRET"] = os.Getenv("ALIYUN_ACCESS_KEY_SECRET")
-		}
-	}
-
-	if kops.CloudProviderID(cluster.Spec.CloudProvider) == kops.CloudProviderAzure {
+	if cluster.Spec.GetCloudProvider() == kops.CloudProviderAzure {
 		env["AZURE_STORAGE_ACCOUNT"] = os.Getenv("AZURE_STORAGE_ACCOUNT")
 		azureEnv := os.Getenv("AZURE_ENVIRONMENT")
 		if azureEnv != "" {
@@ -244,11 +245,7 @@ func (b *BootstrapScriptBuilder) ResourceNodeUp(c *fi.ModelBuilderContext, ig *k
 
 		// Bastions can have AdditionalUserData, but if there isn't any skip this part
 		if len(ig.Spec.AdditionalUserData) == 0 {
-			templateResource, err := NewTemplateResource("nodeup", "", nil, nil)
-			if err != nil {
-				return nil, err
-			}
-			return templateResource, nil
+			return fi.NewStringResource(""), nil
 		}
 	}
 
@@ -312,36 +309,12 @@ func (b *BootstrapScript) Run(c *fi.Context) error {
 		return err
 	}
 
-	functions := template.FuncMap{
-		"NodeUpSourceAmd64": func() string {
-			if b.builder.NodeUpAssets[architectures.ArchitectureAmd64] != nil {
-				return strings.Join(b.builder.NodeUpAssets[architectures.ArchitectureAmd64].Locations, ",")
-			}
-			return ""
-		},
-		"NodeUpSourceHashAmd64": func() string {
-			if b.builder.NodeUpAssets[architectures.ArchitectureAmd64] != nil {
-				return b.builder.NodeUpAssets[architectures.ArchitectureAmd64].Hash.Hex()
-			}
-			return ""
-		},
-		"NodeUpSourceArm64": func() string {
-			if b.builder.NodeUpAssets[architectures.ArchitectureArm64] != nil {
-				return strings.Join(b.builder.NodeUpAssets[architectures.ArchitectureArm64].Locations, ",")
-			}
-			return ""
-		},
-		"NodeUpSourceHashArm64": func() string {
-			if b.builder.NodeUpAssets[architectures.ArchitectureArm64] != nil {
-				return b.builder.NodeUpAssets[architectures.ArchitectureArm64].Hash.Hex()
-			}
-			return ""
-		},
-		"KubeEnv": func() string {
-			return config
-		},
+	var nodeupScript resources.NodeUpScript
+	nodeupScript.NodeUpAssets = b.builder.NodeUpAssets
+	nodeupScript.KubeEnv = config
 
-		"EnvironmentVariables": func() (string, error) {
+	{
+		nodeupScript.EnvironmentVariables = func() (string, error) {
 			env, err := b.buildEnvironmentVariables(c.Cluster)
 			if err != nil {
 				return "", err
@@ -359,13 +332,13 @@ func (b *BootstrapScript) Run(c *fi.Context) error {
 				b.WriteString(fmt.Sprintf("export %s=%s\n", k, env[k]))
 			}
 			return b.String(), nil
-		},
+		}
 
-		"ProxyEnv": func() string {
+		nodeupScript.ProxyEnv = func() (string, error) {
 			return b.createProxyEnv(c.Cluster.Spec.EgressProxy)
-		},
+		}
 
-		"ClusterSpec": func() (string, error) {
+		nodeupScript.ClusterSpec = func() (string, error) {
 			cs := c.Cluster.Spec
 
 			spec := make(map[string]interface{})
@@ -416,38 +389,37 @@ func (b *BootstrapScript) Run(c *fi.Context) error {
 				return "", fmt.Errorf("error converting cluster spec to yaml for inclusion within bootstrap script: %v", err)
 			}
 			return string(content), nil
-		},
-
-		"CompressUserData": func() *bool {
-			return b.ig.Spec.CompressUserData
-		},
-
-		"GzipBase64": func(data string) (string, error) {
-			return gzipBase64(data)
-		},
-
-		"SetSysctls": func() string {
-			// By setting some sysctls early, we avoid broken configurations that prevent nodeup download.
-			// See https://github.com/kubernetes/kops/issues/10206 for details.
-			return setSysctls()
-		},
+		}
 	}
 
-	awsNodeUpTemplate, err := resources.AWSNodeUpTemplate(b.ig)
+	nodeupScript.CompressUserData = fi.BoolValue(b.ig.Spec.CompressUserData)
+
+	// By setting some sysctls early, we avoid broken configurations that prevent nodeup download.
+	// See https://github.com/kubernetes/kops/issues/10206 for details.
+	nodeupScript.SetSysctls = setSysctls()
+
+	nodeupScriptResource, err := nodeupScript.Build()
 	if err != nil {
 		return err
 	}
 
-	templateResource, err := NewTemplateResource("nodeup", awsNodeUpTemplate, functions, nil)
-	if err != nil {
-		return err
-	}
+	b.resource.Resource = fi.FunctionToResource(func() ([]byte, error) {
+		nodeupScript, err := fi.ResourceAsString(nodeupScriptResource)
+		if err != nil {
+			return nil, err
+		}
 
-	b.resource.Resource = templateResource
+		awsUserData, err := resources.AWSMultipartMIME(nodeupScript, b.ig)
+		if err != nil {
+			return nil, err
+		}
+
+		return []byte(awsUserData), nil
+	})
 	return nil
 }
 
-func (b *BootstrapScript) createProxyEnv(ps *kops.EgressProxySpec) string {
+func (b *BootstrapScript) createProxyEnv(ps *kops.EgressProxySpec) (string, error) {
 	var buffer bytes.Buffer
 
 	if ps != nil && ps.HTTPProxy.Host != "" {
@@ -494,27 +466,7 @@ func (b *BootstrapScript) createProxyEnv(ps *kops.EgressProxySpec) string {
 		buffer.WriteString("systemctl daemon-reload\n")
 		buffer.WriteString("systemctl daemon-reexec\n")
 	}
-	return buffer.String()
-}
-
-func gzipBase64(data string) (string, error) {
-	var b bytes.Buffer
-	gz := gzip.NewWriter(&b)
-
-	_, err := gz.Write([]byte(data))
-	if err != nil {
-		return "", err
-	}
-
-	if err = gz.Flush(); err != nil {
-		return "", err
-	}
-
-	if err = gz.Close(); err != nil {
-		return "", err
-	}
-
-	return base64.StdEncoding.EncodeToString(b.Bytes()), nil
+	return buffer.String(), nil
 }
 
 func setSysctls() string {
