@@ -31,7 +31,7 @@ var _ loader.OptionsBuilder = &AWSEBSCSIDriverOptionsBuilder{}
 
 func (b *AWSEBSCSIDriverOptionsBuilder) BuildOptions(o interface{}) error {
 	clusterSpec := o.(*kops.ClusterSpec)
-	if kops.CloudProviderID(clusterSpec.CloudProvider) != kops.CloudProviderAWS {
+	if clusterSpec.GetCloudProvider() != kops.CloudProviderAWS {
 		return nil
 	}
 
@@ -48,7 +48,7 @@ func (b *AWSEBSCSIDriverOptionsBuilder) BuildOptions(o interface{}) error {
 	}
 
 	if c.Version == nil {
-		version := "v1.5.0"
+		version := "v1.8.0"
 		c.Version = fi.String(version)
 	}
 
