@@ -47,11 +47,12 @@ func (b *ContainerdOptionsBuilder) BuildOptions(o interface{}) error {
 		// Set version based on Kubernetes version
 		if fi.StringValue(containerd.Version) == "" {
 			if b.IsKubernetesGTE("1.23") {
-				containerd.Version = fi.String("1.6.4")
-			} else if b.IsKubernetesGTE("1.19") {
-				containerd.Version = fi.String("1.4.12")
+				containerd.Version = fi.String("1.6.8")
+				containerd.Runc = &kops.Runc{
+					Version: fi.String("1.1.4"),
+				}
 			} else {
-				containerd.Version = fi.String("1.3.10")
+				containerd.Version = fi.String("1.4.13")
 			}
 		}
 		// Set default log level to INFO
@@ -86,7 +87,7 @@ func (b *ContainerdOptionsBuilder) BuildOptions(o interface{}) error {
 	}
 
 	if containerd.NvidiaGPU != nil && fi.BoolValue(containerd.NvidiaGPU.Enabled) && containerd.NvidiaGPU.DriverPackage == "" {
-		containerd.NvidiaGPU.DriverPackage = "nvidia-headless-460-server"
+		containerd.NvidiaGPU.DriverPackage = kops.NvidiaDefaultDriverPackage
 	}
 
 	return nil
