@@ -34,11 +34,11 @@ import (
 	"k8s.io/kops/pkg/model"
 	"k8s.io/kops/pkg/wellknownports"
 	"k8s.io/kops/upup/pkg/fi"
-	"k8s.io/kops/upup/pkg/fi/cloudup/aliup"
 	"k8s.io/kops/upup/pkg/fi/cloudup/awsup"
 	"k8s.io/kops/upup/pkg/fi/cloudup/azure"
 	"k8s.io/kops/upup/pkg/fi/cloudup/do"
 	"k8s.io/kops/upup/pkg/fi/cloudup/gce"
+	"k8s.io/kops/upup/pkg/fi/cloudup/hetzner"
 	"k8s.io/kops/upup/pkg/fi/cloudup/openstack"
 	"k8s.io/kops/upup/pkg/fi/fitasks"
 	"k8s.io/kops/util/pkg/env"
@@ -179,8 +179,8 @@ metadata:
   namespace: kube-system
 spec:
   containers:
-  - image: k8s.gcr.io/etcdadm/etcd-manager:v3.0.20220503
-    name: etcd-manager
+  - name: etcd-manager
+    image: registry.k8s.io/etcdadm/etcd-manager:v3.0.20220727
     resources:
       requests:
         cpu: 100m
@@ -348,8 +348,12 @@ func (b *EtcdManagerBuilder) buildPod(etcdCluster kops.EtcdClusterSpec) (*v1.Pod
 		config.LogLevel = int(*etcdCluster.Manager.LogLevel)
 	}
 
+	if etcdCluster.Manager != nil && etcdCluster.Manager.BackupInterval != nil {
+		config.BackupInterval = fi.String(etcdCluster.Manager.BackupInterval.Duration.String())
+	}
+
 	if etcdCluster.Manager != nil && etcdCluster.Manager.DiscoveryPollInterval != nil {
-		config.DiscoveryPollInterval = etcdCluster.Manager.DiscoveryPollInterval
+		config.DiscoveryPollInterval = fi.String(etcdCluster.Manager.DiscoveryPollInterval.Duration.String())
 	}
 
 	{
@@ -372,7 +376,7 @@ func (b *EtcdManagerBuilder) buildPod(etcdCluster kops.EtcdClusterSpec) (*v1.Pod
 	}
 
 	{
-		switch kops.CloudProviderID(b.Cluster.Spec.CloudProvider) {
+		switch b.Cluster.Spec.GetCloudProvider() {
 		case kops.CloudProviderAWS:
 			config.VolumeProvider = "aws"
 
@@ -383,16 +387,6 @@ func (b *EtcdManagerBuilder) buildPod(etcdCluster kops.EtcdClusterSpec) (*v1.Pod
 			}
 			config.VolumeNameTag = awsup.TagNameEtcdClusterPrefix + etcdCluster.Name
 
-		case kops.CloudProviderALI:
-			config.VolumeProvider = "alicloud"
-
-			config.VolumeTag = []string{
-				fmt.Sprintf("kubernetes.io/cluster/%s=owned", b.Cluster.Name),
-				aliup.TagNameEtcdClusterPrefix + etcdCluster.Name,
-				aliup.TagNameRolePrefix + "master=1",
-			}
-			config.VolumeNameTag = aliup.TagNameEtcdClusterPrefix + etcdCluster.Name
-
 		case kops.CloudProviderAzure:
 			config.VolumeProvider = "azure"
 
@@ -427,6 +421,14 @@ func (b *EtcdManagerBuilder) buildPod(etcdCluster kops.EtcdClusterSpec) (*v1.Pod
 			}
 			config.VolumeNameTag = do.TagNameEtcdClusterPrefix + etcdCluster.Name
 
+		case kops.CloudProviderHetzner:
+			config.VolumeProvider = "hetzner"
+
+			config.VolumeTag = []string{
+				fmt.Sprintf("%s=%s", hetzner.TagKubernetesClusterName, b.Cluster.Name),
+				fmt.Sprintf("%s=%s", hetzner.TagKubernetesVolumeRole, etcdCluster.Name),
+			}
+
 		case kops.CloudProviderOpenstack:
 			config.VolumeProvider = "openstack"
 
@@ -438,7 +440,7 @@ func (b *EtcdManagerBuilder) buildPod(etcdCluster kops.EtcdClusterSpec) (*v1.Pod
 			config.VolumeNameTag = openstack.TagNameEtcdClusterPrefix + etcdCluster.Name
 
 		default:
-			return nil, fmt.Errorf("CloudProvider %q not supported with etcd-manager", b.Cluster.Spec.CloudProvider)
+			return nil, fmt.Errorf("CloudProvider %q not supported with etcd-manager", b.Cluster.Spec.GetCloudProvider())
 		}
 	}
 
@@ -466,39 +468,10 @@ func (b *EtcdManagerBuilder) buildPod(etcdCluster kops.EtcdClusterSpec) (*v1.Pod
 			},
 		}
 
-		// TODO: Use helper function here
-		container.VolumeMounts = append(container.VolumeMounts, v1.VolumeMount{
-			Name:      "varlogetcd",
-			MountPath: "/var/log/etcd.log",
-			ReadOnly:  false,
-		})
-		hostPathFileOrCreate := v1.HostPathFileOrCreate
-		pod.Spec.Volumes = append(pod.Spec.Volumes, v1.Volume{
-			Name: "varlogetcd",
-			VolumeSource: v1.VolumeSource{
-				HostPath: &v1.HostPathVolumeSource{
-					Path: logFile,
-					Type: &hostPathFileOrCreate,
-				},
-			},
-		})
+		kubemanifest.AddHostPathMapping(pod, container, "varlogetcd", "/var/log/etcd.log").WithReadWrite().WithType(v1.HostPathFileOrCreate).WithHostPath(logFile)
 
 		if fi.BoolValue(b.Cluster.Spec.UseHostCertificates) {
-			container.VolumeMounts = append(container.VolumeMounts, v1.VolumeMount{
-				Name:      "etc-ssl-certs",
-				MountPath: "/etc/ssl/certs",
-				ReadOnly:  true,
-			})
-			hostPathDirectoryOrCreate := v1.HostPathDirectoryOrCreate
-			pod.Spec.Volumes = append(pod.Spec.Volumes, v1.Volume{
-				Name: "etc-ssl-certs",
-				VolumeSource: v1.VolumeSource{
-					HostPath: &v1.HostPathVolumeSource{
-						Path: "/etc/ssl/certs",
-						Type: &hostPathDirectoryOrCreate,
-					},
-				},
-			})
+			kubemanifest.AddHostPathMapping(pod, container, "etc-ssl-certs", "/etc/ssl/certs").WithType(v1.HostPathDirectoryOrCreate)
 		}
 	}
 
@@ -561,6 +534,7 @@ type config struct {
 	QuarantineClientUrls  string   `flag:"quarantine-client-urls"`
 	ClusterName           string   `flag:"cluster-name"`
 	BackupStore           string   `flag:"backup-store"`
+	BackupInterval        *string  `flag:"backup-interval"`
 	DataDir               string   `flag:"data-dir"`
 	VolumeProvider        string   `flag:"volume-provider"`
 	VolumeTag             []string `flag:"volume-tag,repeat"`
