@@ -61,7 +61,6 @@ func (b *KubeletOptionsBuilder) BuildOptions(o interface{}) error {
 	clusterSpec.Kubelet.PodManifestPath = "/etc/kubernetes/manifests"
 	clusterSpec.Kubelet.LogLevel = fi.Int32(2)
 	clusterSpec.Kubelet.ClusterDomain = clusterSpec.ClusterDNSDomain
-	clusterSpec.Kubelet.NonMasqueradeCIDR = clusterSpec.NonMasqueradeCIDR
 
 	// AllowPrivileged is deprecated and removed in v1.14.
 	// See https://github.com/kubernetes/kubernetes/pull/71835
@@ -123,7 +122,7 @@ func (b *KubeletOptionsBuilder) BuildOptions(o interface{}) error {
 		clusterSpec.MasterKubelet.HairpinMode = "none"
 	}
 
-	cloudProvider := kops.CloudProviderID(clusterSpec.CloudProvider)
+	cloudProvider := clusterSpec.GetCloudProvider()
 
 	clusterSpec.Kubelet.CgroupRoot = "/"
 
@@ -148,12 +147,12 @@ func (b *KubeletOptionsBuilder) BuildOptions(o interface{}) error {
 
 	}
 
-	if cloudProvider == kops.CloudProviderOpenstack {
-		clusterSpec.Kubelet.CloudProvider = "openstack"
+	if cloudProvider == kops.CloudProviderHetzner {
+		clusterSpec.Kubelet.CloudProvider = "external"
 	}
 
-	if cloudProvider == kops.CloudProviderALI {
-		clusterSpec.Kubelet.CloudProvider = "alicloud"
+	if cloudProvider == kops.CloudProviderOpenstack {
+		clusterSpec.Kubelet.CloudProvider = "openstack"
 	}
 
 	if cloudProvider == kops.CloudProviderAzure {
@@ -169,17 +168,16 @@ func (b *KubeletOptionsBuilder) BuildOptions(o interface{}) error {
 		if networking == nil {
 			return fmt.Errorf("no networking mode set")
 		}
-		if UsesKubenet(networking) {
-			clusterSpec.Kubelet.NetworkPluginName = "kubenet"
-
-			// AWS MTU is 9001
+		if UsesKubenet(networking) && b.IsKubernetesLT("1.24") {
+			clusterSpec.Kubelet.NetworkPluginName = fi.String("kubenet")
 			clusterSpec.Kubelet.NetworkPluginMTU = fi.Int32(9001)
+			clusterSpec.Kubelet.NonMasqueradeCIDR = fi.String(clusterSpec.NonMasqueradeCIDR)
 		}
 	}
 
 	// Prevent image GC from pruning the pause image
 	// https://github.com/kubernetes/enhancements/tree/master/keps/sig-node/2040-kubelet-cri#pinned-images
-	image := "k8s.gcr.io/pause:3.6"
+	image := "registry.k8s.io/pause:3.6"
 	var err error
 	if image, err = b.AssetBuilder.RemapImage(image); err != nil {
 		return err
