@@ -60,6 +60,10 @@ func (b *NodeTerminationHandlerOptionsBuilder) BuildOptions(o interface{}) error
 		nth.EnableSQSTerminationDraining = fi.Bool(false)
 	}
 
+	if nth.ExcludeFromLoadBalancers == nil {
+		nth.ExcludeFromLoadBalancers = fi.Bool(true)
+	}
+
 	if nth.ManagedASGTag == nil {
 		nth.ManagedASGTag = fi.String("aws-node-termination-handler/managed")
 	}
@@ -74,5 +78,9 @@ func (b *NodeTerminationHandlerOptionsBuilder) BuildOptions(o interface{}) error
 		nth.MemoryRequest = &defaultMemoryRequest
 	}
 
+	if nth.Version == nil {
+		nth.Version = fi.String("v1.16.5")
+	}
+
 	return nil
 }
