@@ -35,6 +35,7 @@ import (
 
 	"k8s.io/apimachinery/pkg/util/sets"
 	"k8s.io/klog/v2"
+
 	"k8s.io/kops/pkg/apis/kops"
 	"k8s.io/kops/pkg/apis/kops/model"
 	"k8s.io/kops/pkg/util/stringorslice"
@@ -423,8 +424,8 @@ func (r *NodeRoleMaster) BuildAWSPolicy(b *PolicyBuilder) (*Policy, error) {
 			}
 		}
 
-		if b.Cluster.Spec.AWSLoadBalancerController != nil && fi.BoolValue(b.Cluster.Spec.AWSLoadBalancerController.Enabled) {
-			AddAWSLoadbalancerControllerPermissions(p, b.Partition)
+		if c := b.Cluster.Spec.AWSLoadBalancerController; c != nil && fi.BoolValue(b.Cluster.Spec.AWSLoadBalancerController.Enabled) {
+			AddAWSLoadbalancerControllerPermissions(p, c.EnableWAF, c.EnableWAFv2, c.EnableShield)
 		}
 
 		var useStaticInstanceList bool
@@ -873,8 +874,6 @@ func AddLegacyCCMPermissions(p *Policy) {
 		"elasticloadbalancing:AddTags",
 		"elasticloadbalancing:RegisterTargets",
 		"elasticloadbalancing:CreateListener",
-		"elasticloadbalancing:DeleteListener",
-		"elasticloadbalancing:ModifyListener",
 		"ec2:DescribeVolumes",
 		"ec2:ModifyInstanceAttribute",
 		"ec2:ModifyVolume",
@@ -924,7 +923,9 @@ func AddCCMPermissions(p *Policy, cloudRoutes bool) {
 		"elasticloadbalancing:RegisterInstancesWithLoadBalancer",
 		"elasticloadbalancing:SetLoadBalancerPoliciesForBackendServer",
 		"elasticloadbalancing:AddTags",
+		"elasticloadbalancing:DeleteListener",
 		"elasticloadbalancing:DeleteTargetGroup",
+		"elasticloadbalancing:ModifyListener",
 		"elasticloadbalancing:ModifyTargetGroup",
 		"elasticloadbalancing:RegisterTargets",
 		"elasticloadbalancing:DeregisterTargets",
@@ -954,8 +955,8 @@ func AddCCMPermissions(p *Policy, cloudRoutes bool) {
 	}
 }
 
-// AddAWSLoadbalancerControllerPermissions adds the permissions needed for the aws load balancer controller to the givnen policy
-func AddAWSLoadbalancerControllerPermissions(p *Policy, partition string) {
+// AddAWSLoadbalancerControllerPermissions adds the permissions needed for the AWS Load Balancer Controller to the givnen policy
+func AddAWSLoadbalancerControllerPermissions(p *Policy, enableWAF, enableWAFv2, enableShield bool) {
 	p.unconditionalAction.Insert(
 		"acm:DescribeCertificate",
 		"acm:ListCertificates",
@@ -979,23 +980,53 @@ func AddAWSLoadbalancerControllerPermissions(p *Policy, partition string) {
 		"elasticloadbalancing:DescribeTargetGroupAttributes",
 		"elasticloadbalancing:DescribeTargetHealth",
 	)
+	if enableWAF {
+		p.unconditionalAction.Insert(
+			"elasticloadbalancing:SetWebACL",
+			"waf-regional:AssociateWebACL",
+			"waf-regional:DisassociateWebACL",
+			"waf-regional:GetWebACL",
+			"waf-regional:GetWebACLForResource",
+		)
+	}
+	if enableWAFv2 {
+		p.unconditionalAction.Insert(
+			"elasticloadbalancing:SetWebACL",
+			"wafv2:AssociateWebACL",
+			"wafv2:DisassociateWebACL",
+			"wafv2:GetWebACL",
+			"wafv2:GetWebACLForResource",
+		)
+	}
+
+	if enableShield {
+		p.unconditionalAction.Insert(
+			"shield:GetSubscriptionState",
+			"shield:DescribeProtection",
+			"shield:CreateProtection",
+			"shield:DeleteProtection",
+		)
+	}
+
 	p.clusterTaggedAction.Insert(
 		"ec2:AuthorizeSecurityGroupIngress", // aws.go
 		"ec2:DeleteSecurityGroup",           // aws.go
 		"ec2:RevokeSecurityGroupIngress",    // aws.go
 
+		"elasticloadbalancing:AddListenerCertificates",
 		"elasticloadbalancing:AddTags",
 		"elasticloadbalancing:DeleteListener",
 		"elasticloadbalancing:DeleteLoadBalancer",
-		"elasticloadbalancing:DeleteTargetGroup",
 		"elasticloadbalancing:DeleteRule",
+		"elasticloadbalancing:DeleteTargetGroup",
 		"elasticloadbalancing:DeregisterTargets",
-		"elasticloadbalancing:ModifyLoadBalancerAttributes",
 		"elasticloadbalancing:ModifyListener",
+		"elasticloadbalancing:ModifyLoadBalancerAttributes",
 		"elasticloadbalancing:ModifyRule",
 		"elasticloadbalancing:ModifyTargetGroup",
 		"elasticloadbalancing:ModifyTargetGroupAttributes",
 		"elasticloadbalancing:RegisterTargets",
+		"elasticloadbalancing:RemoveListenerCertificates",
 		"elasticloadbalancing:RemoveTags",
 		"elasticloadbalancing:SetIpAddressType",
 		"elasticloadbalancing:SetSecurityGroups",
@@ -1114,6 +1145,14 @@ func AddDNSControllerPermissions(b *PolicyBuilder, p *Policy) {
 	})
 }
 
+// AddKubeRouterPermissions adds IAM permissions used by kube-router
+// for disabling the source/destination check on EC2 instances.
+func AddKubeRouterPermissions(b *PolicyBuilder, p *Policy) {
+	p.clusterTaggedAction.Insert(
+		"ec2:ModifyInstanceAttribute",
+	)
+}
+
 func addKMSIAMPolicies(p *Policy, resource stringorslice.StringOrSlice) {
 	// TODO could use "kms:ViaService" Condition Key here?
 	p.unconditionalAction.Insert(
