@@ -19,7 +19,7 @@ source "${REPO_ROOT}"/tests/e2e/scenarios/lib/common.sh
 
 function haveds() {
 	local ds=0
-	kubectl get ds -n kube-system aws-node-termination-handler || ds=$?
+	kubectl get ds -n kube-system aws-node-termination-handler --show-labels || ds=$?
 	return $ds
 }
 
@@ -39,10 +39,13 @@ ${KUBETEST2} \
     --create-args="$ARGS"
 
 
-haveds
+if ! haveds; then
+  echo "Expected aws-node-termination-handler to exist"
+  exit 1
+fi
 
 # Upgrade to a version that should adopt existing resources and apply the change below
-KOPS=$(kops-acquire-latest)
+kops-acquire-latest
 
 cp "${KOPS}" "${WORKSPACE}/kops"
 
@@ -53,11 +56,14 @@ kops edit cluster "${CLUSTER_NAME}" "--set=cluster.spec.nodeTerminationHandler.e
 kops update cluster --allow-kops-downgrade
 kops update cluster --yes --allow-kops-downgrade
 
-# wait for channels to deploy
-sleep 90s
+# Rolling-upgrade is needed so we get the new channels binary that supports prune
+kops rolling-update cluster --instance-group-roles=master --yes
 
 # just make sure pods are ready
 kops validate cluster --wait=5m
 
 # We should no longer have a daemonset called aws-node-termination-handler
-haveds && exit 1
\ No newline at end of file
+if haveds; then
+  echo "Expected aws-node-termination-handler to have been pruned"
+  exit 1
+fi
\ No newline at end of file
