@@ -32,16 +32,31 @@ fi
 export KOPS_BASE_URL
 
 echo "Cleaning up any leaked resources from previous cluster"
-KOPS_BASE_URL=$(kops-base-from-marker "${KOPS_VERSION_B}")
-KOPS_BASE_URL_B="${KOPS_BASE_URL}"
-KOPS_B=$(kops-download-from-base)
+# For KOPS_VERSION_B, the value "latest" means build of the tree
+if [[ "${KOPS_VERSION_B}" == "latest" ]]; then
+  kops-acquire-latest
+  KOPS_BASE_URL_B="${KOPS_BASE_URL}"
+  KOPS_B="${KOPS}"
+else
+  KOPS_BASE_URL=$(kops-base-from-marker "${KOPS_VERSION_B}")
+  KOPS_BASE_URL_B="${KOPS_BASE_URL}"
+  KOPS_B=$(kops-download-from-base)
+fi
+
 ${KUBETEST2} \
 		--down \
 		--kops-binary-path="${KOPS_B}" || echo "kubetest2 down failed"
 
-KOPS_BASE_URL=$(kops-base-from-marker "${KOPS_VERSION_A}")
-KOPS_A=$(kops-download-from-base)
-KOPS="${KOPS_A}"
+# First kOps version may be a released version. If so, it is prefixed with v
+if [[ "${KOPS_VERSION_A:0:1}" == "v" ]]; then
+  KOPS_BASE_URL=""
+  KOPS_A=$(kops-download-release "$KOPS_VERSION_A")
+  KOPS="${KOPS_A}"
+else
+  KOPS_BASE_URL=$(kops-base-from-marker "${KOPS_VERSION_A}")
+  KOPS_A=$(kops-download-from-base)
+  KOPS="${KOPS_A}"
+fi
 
 ${KUBETEST2} \
 		--up \
@@ -89,11 +104,14 @@ cp "${KOPS_B}" "${WORKSPACE}/kops"
 
 "${KOPS_B}" export kubecfg --name "${CLUSTER_NAME}" --admin
 
+if [[ -n ${KOPS_SKIP_E2E:-} ]]; then
+  exit
+fi
+
 ${KUBETEST2} \
 		--cloud-provider="${CLOUD_PROVIDER}" \
 		--kops-binary-path="${KOPS}" \
 		--test=kops \
 		-- \
 		--test-package-version="${K8S_VERSION_B}" \
-		--parallel 25 \
-		--skip-regex="\[Slow\]|\[Serial\]|\[Disruptive\]|\[Flaky\]|\[Feature:.+\]|\[HPA\]|Dashboard|RuntimeClass|RuntimeHandler|TCP.CLOSE_WAIT|Projected.configMap.optional.updates|Invalid.AWS.KMS.key|Volume.limits.should.verify.that.all.nodes.have.volume.limits"
+		--parallel 25
