@@ -25,8 +25,8 @@ import (
 	"strings"
 
 	"golang.org/x/oauth2/google"
+	"google.golang.org/api/cloudresourcemanager/v1"
 	compute "google.golang.org/api/compute/v1"
-	"google.golang.org/api/iam/v1"
 	oauth2 "google.golang.org/api/oauth2/v2"
 	"google.golang.org/api/storage/v1"
 	"k8s.io/klog/v2"
@@ -41,7 +41,7 @@ type GCECloud interface {
 	fi.Cloud
 	Compute() ComputeClient
 	Storage() *storage.Service
-	IAM() *iam.Service
+	IAM() IamClient
 	CloudDNS() DNSClient
 	Project() string
 	WaitForOp(op *compute.Operation) error
@@ -50,14 +50,20 @@ type GCECloud interface {
 
 	// ServiceAccount returns the email for the service account that the instances will run under
 	ServiceAccount() (string, error)
+
+	// CloudResourceManager returns the client for the cloudresourcemanager API
+	CloudResourceManager() *cloudresourcemanager.Service
 }
 
 type gceCloudImplementation struct {
 	compute *computeClientImpl
 	storage *storage.Service
-	iam     *iam.Service
+	iam     *iamClientImpl
 	dns     *dnsClientImpl
 
+	// cloudResourceManager is the client for the cloudresourcemanager API
+	cloudResourceManager *cloudresourcemanager.Service
+
 	region  string
 	project string
 
@@ -132,7 +138,7 @@ func NewGCECloud(region string, project string, labels map[string]string) (GCECl
 	}
 	c.storage = storageService
 
-	iamService, err := iam.NewService(ctx)
+	iamService, err := newIamClientImpl(ctx)
 	if err != nil {
 		return nil, fmt.Errorf("error building IAM API client: %v", err)
 	}
@@ -144,6 +150,12 @@ func NewGCECloud(region string, project string, labels map[string]string) (GCECl
 	}
 	c.dns = dnsClient
 
+	cloudResourceManager, err := cloudresourcemanager.NewService(ctx)
+	if err != nil {
+		return nil, fmt.Errorf("error building cloudresourcemanager API client: %w", err)
+	}
+	c.cloudResourceManager = cloudResourceManager
+
 	CacheGCECloudInstance(region, project, c)
 
 	{
@@ -189,15 +201,20 @@ func (c *gceCloudImplementation) Storage() *storage.Service {
 }
 
 // IAM returns the IAM client
-func (c *gceCloudImplementation) IAM() *iam.Service {
+func (c *gceCloudImplementation) IAM() IamClient {
 	return c.iam
 }
 
-// NameService returns the DNS client
+// CloudDNS returns the DNS client
 func (c *gceCloudImplementation) CloudDNS() DNSClient {
 	return c.dns
 }
 
+// CloudResourceManager returns the client for the cloudresourcemanager API
+func (c *gceCloudImplementation) CloudResourceManager() *cloudresourcemanager.Service {
+	return c.cloudResourceManager
+}
+
 // Region returns private struct element region.
 func (c *gceCloudImplementation) Region() string {
 	return c.region
@@ -287,10 +304,18 @@ func (c *gceCloudImplementation) GetApiIngressStatus(cluster *kops.Cluster) ([]f
 	var ingresses []fi.ApiIngressStatus
 
 	// Note that this must match GCEModelContext::NameForForwardingRule
-	name := SafeObjectName("api", cluster.ObjectMeta.Name)
+	name := ClusterSuffixedName("api", cluster.ObjectMeta.Name, 63)
 
 	klog.V(2).Infof("Querying GCE to find ForwardingRules for API (%q)", name)
-	forwardingRule, err := c.compute.ForwardingRules().Get(c.project, c.region, name)
+	// These are the ingress rules, so we search for them in the network project.
+	_, project, err := ParseNameAndProjectFromNetworkID(cluster.Spec.NetworkID)
+	if err != nil {
+		return nil, err
+	} else if project == "" {
+		project = c.Project()
+	}
+
+	forwardingRule, err := c.compute.ForwardingRules().Get(project, c.region, name)
 	if err != nil {
 		if !IsNotFound(err) {
 			forwardingRule = nil
@@ -360,3 +385,22 @@ func (c *gceCloudImplementation) getTokenInfo(ctx context.Context) (*oauth2.Toke
 
 	return tokenInfo, nil
 }
+
+// SplitServiceAccountEmail splits service account email
+func SplitServiceAccountEmail(email string) (string, string, error) {
+	accountID := ""
+	projectID := ""
+
+	tokens := strings.Split(email, "@")
+	if len(tokens) == 2 {
+		accountID = tokens[0]
+		if strings.HasSuffix(tokens[1], ".iam.gserviceaccount.com") {
+			projectID = strings.TrimSuffix(tokens[1], ".iam.gserviceaccount.com")
+		}
+	}
+
+	if accountID == "" || projectID == "" {
+		return "", "", fmt.Errorf("unexpected format for ServiceAccount email %q", email)
+	}
+	return accountID, projectID, nil
+}
