@@ -65,7 +65,7 @@ type InstanceTemplate struct {
 	AliasIPRanges map[string]string
 
 	Scopes          []string
-	ServiceAccounts []string
+	ServiceAccounts []*ServiceAccount
 
 	Metadata    map[string]fi.Resource
 	MachineType *string
@@ -75,6 +75,8 @@ type InstanceTemplate struct {
 
 	// ID is the actual name
 	ID *string
+
+	GuestAccelerators []AcceleratorConfig
 }
 
 var (
@@ -164,7 +166,9 @@ func (e *InstanceTemplate) Find(c *fi.Context) (*InstanceTemplate, error) {
 			for _, scope := range serviceAccount.Scopes {
 				actual.Scopes = append(actual.Scopes, scopeToShortForm(scope))
 			}
-			actual.ServiceAccounts = append(actual.ServiceAccounts, serviceAccount.Email)
+			actual.ServiceAccounts = append(actual.ServiceAccounts, &ServiceAccount{
+				Email: &serviceAccount.Email,
+			})
 		}
 
 		// When we deal with additional disks (local disks), we'll need to map them like this...
@@ -209,6 +213,14 @@ func (e *InstanceTemplate) Find(c *fi.Context) (*InstanceTemplate, error) {
 		// System fields
 		actual.Lifecycle = e.Lifecycle
 
+		actual.GuestAccelerators = []AcceleratorConfig{}
+		for _, accelerator := range p.GuestAccelerators {
+			actual.GuestAccelerators = append(actual.GuestAccelerators, AcceleratorConfig{
+				AcceleratorCount: accelerator.AcceleratorCount,
+				AcceleratorType:  accelerator.AcceleratorType,
+			})
+		}
+
 		return actual, nil
 	}
 
@@ -237,6 +249,7 @@ func (e *InstanceTemplate) mapToGCE(project string, region string) (*compute.Ins
 		scheduling = &compute.Scheduling{
 			AutomaticRestart:  fi.Bool(false),
 			OnHostMaintenance: "TERMINATE",
+			ProvisioningModel: "STANDARD", // TODO: Support Spot?
 			Preemptible:       true,
 		}
 	} else {
@@ -244,10 +257,16 @@ func (e *InstanceTemplate) mapToGCE(project string, region string) (*compute.Ins
 			AutomaticRestart: fi.Bool(true),
 			// TODO: Migrate or terminate?
 			OnHostMaintenance: "MIGRATE",
+			ProvisioningModel: "STANDARD",
 			Preemptible:       false,
 		}
 	}
 
+	if len(e.GuestAccelerators) > 0 {
+		// Instances with accelerators cannot be migrated.
+		scheduling.OnHostMaintenance = "TERMINATE"
+	}
+
 	var disks []*compute.AttachedDisk
 	disks = append(disks, &compute.AttachedDisk{
 		Kind: "compute#attachedDisk",
@@ -272,9 +291,15 @@ func (e *InstanceTemplate) mapToGCE(project string, region string) (*compute.Ins
 	}
 
 	var networkInterfaces []*compute.NetworkInterface
+
+	networkProject := project
+	if e.Network.Project != nil {
+		networkProject = *e.Network.Project
+	}
+
 	ni := &compute.NetworkInterface{
 		Kind:    "compute#networkInterface",
-		Network: e.Network.URL(project),
+		Network: e.Network.URL(networkProject),
 	}
 	if fi.BoolValue(e.HasExternalIP) {
 		ni.AccessConfigs = []*compute.AccessConfig{
@@ -287,7 +312,7 @@ func (e *InstanceTemplate) mapToGCE(project string, region string) (*compute.Ins
 	}
 
 	if e.Subnet != nil {
-		ni.Subnetwork = e.Subnet.URL(project, region)
+		ni.Subnetwork = e.Subnet.URL(networkProject, region)
 	}
 	if e.AliasIPRanges != nil {
 		for k, v := range e.AliasIPRanges {
@@ -306,25 +331,14 @@ func (e *InstanceTemplate) mapToGCE(project string, region string) (*compute.Ins
 			scopes = append(scopes, s)
 		}
 	}
-	serviceAccounts := []*compute.ServiceAccount{
-		{
-			Email:  e.ServiceAccounts[0],
+
+	var serviceAccounts []*compute.ServiceAccount
+	for _, sa := range e.ServiceAccounts {
+		serviceAccounts = append(serviceAccounts, &compute.ServiceAccount{
+			Email:  fi.StringValue(sa.Email),
 			Scopes: scopes,
-		},
+		})
 	}
-	// if e.ServiceAccounts != nil {
-	//	for _, s := range e.ServiceAccounts {
-	//		serviceAccounts = append(serviceAccounts, &compute.ServiceAccount{
-	//			Email:  s,
-	//			Scopes: scopes,
-	//		})
-	//	}
-	// } else {
-	//	serviceAccounts = append(serviceAccounts, &compute.ServiceAccount{
-	//		Email:  "default",
-	//		Scopes: scopes,
-	//	})
-	// }
 
 	var metadataItems []*compute.MetadataItems
 	for key, r := range e.Metadata {
@@ -338,6 +352,17 @@ func (e *InstanceTemplate) mapToGCE(project string, region string) (*compute.Ins
 		})
 	}
 
+	var accelerators []*compute.AcceleratorConfig
+	if len(e.GuestAccelerators) > 0 {
+		accelerators = []*compute.AcceleratorConfig{}
+		for _, accelerator := range e.GuestAccelerators {
+			accelerators = append(accelerators, &compute.AcceleratorConfig{
+				AcceleratorCount: accelerator.AcceleratorCount,
+				AcceleratorType:  accelerator.AcceleratorType,
+			})
+		}
+	}
+
 	i := &compute.InstanceTemplate{
 		Kind: "compute#instanceTemplate",
 		Properties: &compute.InstanceProperties{
@@ -345,6 +370,8 @@ func (e *InstanceTemplate) mapToGCE(project string, region string) (*compute.Ins
 
 			Disks: disks,
 
+			GuestAccelerators: accelerators,
+
 			MachineType: *e.MachineType,
 
 			Metadata: &compute.Metadata{
@@ -454,7 +481,7 @@ type terraformInstanceTemplate struct {
 	NamePrefix            string                                   `cty:"name_prefix"`
 	CanIPForward          bool                                     `cty:"can_ip_forward"`
 	MachineType           string                                   `cty:"machine_type"`
-	ServiceAccount        *terraformServiceAccount                 `cty:"service_account"`
+	ServiceAccounts       []*terraformTemplateServiceAccount       `cty:"service_account"`
 	Scheduling            *terraformScheduling                     `cty:"scheduling"`
 	Disks                 []*terraformInstanceTemplateAttachedDisk `cty:"disk"`
 	Labels                map[string]string                        `cty:"labels"`
@@ -462,11 +489,12 @@ type terraformInstanceTemplate struct {
 	Metadata              map[string]*terraformWriter.Literal      `cty:"metadata"`
 	MetadataStartupScript *terraformWriter.Literal                 `cty:"metadata_startup_script"`
 	Tags                  []string                                 `cty:"tags"`
+	GuestAccelerator      []*terraformGuestAccelerator             `cty:"guest_accelerator"`
 }
 
-type terraformServiceAccount struct {
-	Email  string   `cty:"email"`
-	Scopes []string `cty:"scopes"`
+type terraformTemplateServiceAccount struct {
+	Email  *terraformWriter.Literal `cty:"email"`
+	Scopes []string                 `cty:"scopes"`
 }
 
 type terraformScheduling struct {
@@ -501,6 +529,11 @@ type terraformAccessConfig struct {
 	NatIP *terraformWriter.Literal `cty:"nat_ip"`
 }
 
+type terraformGuestAccelerator struct {
+	Type  string `cty:"type"`
+	Count int64  `cty:"count"`
+}
+
 func addNetworks(network *Network, subnet *Subnet, networkInterfaces []*compute.NetworkInterface) []*terraformNetworkInterface {
 	ni := make([]*terraformNetworkInterface, 0)
 	for _, g := range networkInterfaces {
@@ -546,22 +579,23 @@ func addMetadata(target *terraform.TerraformTarget, name string, metadata *compu
 	return m, nil
 }
 
-func addServiceAccounts(serviceAccounts []*compute.ServiceAccount) *terraformServiceAccount {
-	// there's an inconsistency here- GCP only lets you have one service account per VM
-	// terraform gets it right, but the golang api doesn't. womp womp :(
-	if len(serviceAccounts) != 1 {
-		klog.Fatal("Instances can only have 1 service account assigned.")
+func mapServiceAccountsToTerraform(serviceAccounts []*ServiceAccount, saScopes []string) []*terraformTemplateServiceAccount {
+	var scopes []string
+	for _, s := range saScopes {
+		s = scopeToLongForm(s)
+		scopes = append(scopes, s)
 	}
-	klog.Infof("adding csa: %v", serviceAccounts[0].Email)
-	csa := serviceAccounts[0]
-	tsa := &terraformServiceAccount{
-		Email:  csa.Email,
-		Scopes: csa.Scopes,
+	// Note that GCE currently only allows one service account per VM,
+	// but the model in both the API and terraform allows more.
+	var out []*terraformTemplateServiceAccount
+	for _, serviceAccount := range serviceAccounts {
+		tsa := &terraformTemplateServiceAccount{
+			Email:  serviceAccount.TerraformLink(),
+			Scopes: scopes,
+		}
+		out = append(out, tsa)
 	}
-	// for _, scope := range csa.Scopes {
-	//	tsa.Scopes = append(tsa.Scopes, scope)
-	// }
-	return tsa
+	return out
 }
 
 func (_ *InstanceTemplate) RenderTerraform(t *terraform.TerraformTarget, a, e, changes *InstanceTemplate) error {
@@ -583,7 +617,7 @@ func (_ *InstanceTemplate) RenderTerraform(t *terraform.TerraformTarget, a, e, c
 	tf.Labels = i.Properties.Labels
 	tf.Tags = i.Properties.Tags.Items
 
-	tf.ServiceAccount = addServiceAccounts(i.Properties.ServiceAccounts)
+	tf.ServiceAccounts = mapServiceAccountsToTerraform(e.ServiceAccounts, e.Scopes)
 
 	for _, d := range i.Properties.Disks {
 		tfd := &terraformInstanceTemplateAttachedDisk{
@@ -618,6 +652,16 @@ func (_ *InstanceTemplate) RenderTerraform(t *terraform.TerraformTarget, a, e, c
 		}
 	}
 
+	if len(i.Properties.GuestAccelerators) > 0 {
+		tf.GuestAccelerator = []*terraformGuestAccelerator{}
+		for _, accelerator := range i.Properties.GuestAccelerators {
+			tf.GuestAccelerator = append(tf.GuestAccelerator, &terraformGuestAccelerator{
+				Count: accelerator.AcceleratorCount,
+				Type:  accelerator.AcceleratorType,
+			})
+		}
+	}
+
 	return t.RenderResource("google_compute_instance_template", name, tf)
 }
 
