@@ -27,6 +27,7 @@ import (
 	v1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/util/sets"
 	"k8s.io/klog/v2"
+
 	"k8s.io/kops"
 	api "k8s.io/kops/pkg/apis/kops"
 	"k8s.io/kops/pkg/apis/kops/model"
@@ -35,7 +36,6 @@ import (
 	"k8s.io/kops/pkg/featureflag"
 	"k8s.io/kops/pkg/zones"
 	"k8s.io/kops/upup/pkg/fi"
-	"k8s.io/kops/upup/pkg/fi/cloudup/aliup"
 	"k8s.io/kops/upup/pkg/fi/cloudup/awsup"
 	"k8s.io/kops/upup/pkg/fi/cloudup/azure"
 	"k8s.io/kops/upup/pkg/fi/cloudup/gce"
@@ -147,6 +147,9 @@ type NewClusterOptions struct {
 	// APISSLCertificate is the SSL certificate to use for the API loadbalancer.
 	// Currently only supported in AWS.
 	APISSLCertificate string
+
+	// InstanceManager specifies which manager to use for managing instances.
+	InstanceManager string
 }
 
 func (o *NewClusterOptions) InitDefaults() {
@@ -156,6 +159,7 @@ func (o *NewClusterOptions) InitDefaults() {
 	o.Networking = "kubenet"
 	o.Topology = api.TopologyPublic
 	o.DNSType = string(api.DNSTypePublic)
+	o.InstanceManager = "cloudgroups"
 }
 
 type NewClusterResult struct {
@@ -240,23 +244,57 @@ func NewCluster(opt *NewClusterOptions, clientset simple.Clientset) (*NewCluster
 	allZones.Insert(opt.Zones...)
 	allZones.Insert(opt.MasterZones...)
 
-	cluster.Spec.CloudProvider = opt.CloudProvider
-	if cluster.Spec.CloudProvider == "" {
+	if opt.CloudProvider == "" {
 		for _, zone := range allZones.List() {
 			cloud, known := zones.GuessCloudForZone(zone)
 			if known {
 				klog.Infof("Inferred %q cloud provider from zone %q", cloud, zone)
-				cluster.Spec.CloudProvider = string(cloud)
+				opt.CloudProvider = string(cloud)
 				break
 			}
 		}
-		if cluster.Spec.CloudProvider == "" {
+		if opt.CloudProvider == "" {
 			if allZones.Len() == 0 {
 				return nil, fmt.Errorf("must specify --zones or --cloud")
 			}
 			return nil, fmt.Errorf("unable to infer cloud provider from zones. pass in the cloud provider explicitly using --cloud")
 		}
 	}
+	switch api.CloudProviderID(opt.CloudProvider) {
+	case api.CloudProviderAWS:
+		cluster.Spec.CloudProvider.AWS = &api.AWSSpec{}
+	case api.CloudProviderAzure:
+		cluster.Spec.CloudProvider.Azure = &api.AzureSpec{
+			SubscriptionID:    opt.AzureSubscriptionID,
+			TenantID:          opt.AzureTenantID,
+			ResourceGroupName: opt.AzureResourceGroupName,
+			RouteTableName:    opt.AzureRouteTableName,
+			AdminUser:         opt.AzureAdminUser,
+		}
+	case api.CloudProviderDO:
+		cluster.Spec.CloudProvider.DO = &api.DOSpec{}
+	case api.CloudProviderGCE:
+		cluster.Spec.CloudProvider.GCE = &api.GCESpec{}
+	case api.CloudProviderHetzner:
+		cluster.Spec.CloudProvider.Hetzner = &api.HetznerSpec{}
+	case api.CloudProviderOpenstack:
+		cluster.Spec.CloudProvider.Openstack = &api.OpenstackSpec{
+			Router: &api.OpenstackRouter{
+				ExternalNetwork: fi.String(opt.OpenstackExternalNet),
+			},
+			BlockStorage: &api.OpenstackBlockStorageConfig{
+				Version:  fi.String("v3"),
+				IgnoreAZ: fi.Bool(opt.OpenstackStorageIgnoreAZ),
+			},
+			Monitor: &api.OpenstackMonitor{
+				Delay:      fi.String("15s"),
+				Timeout:    fi.String("10s"),
+				MaxRetries: fi.Int(3),
+			},
+		}
+	default:
+		return nil, fmt.Errorf("unsupported cloud provider %s", opt.CloudProvider)
+	}
 
 	if opt.DiscoveryStore != "" {
 		discoveryPath, err := vfs.Context.BuildVfsPath(opt.DiscoveryStore)
@@ -266,7 +304,7 @@ func NewCluster(opt *NewClusterOptions, clientset simple.Clientset) (*NewCluster
 		cluster.Spec.ServiceAccountIssuerDiscovery = &api.ServiceAccountIssuerDiscoveryConfig{
 			DiscoveryStore: discoveryPath.Join(cluster.Name).Path(),
 		}
-		if cluster.Spec.CloudProvider == string(api.CloudProviderAWS) {
+		if cluster.Spec.GetCloudProvider() == api.CloudProviderAWS {
 			cluster.Spec.ServiceAccountIssuerDiscovery.EnableAWSOIDCProvider = true
 			cluster.Spec.IAM.UseServiceAccountExternalPermissions = fi.Bool(true)
 		}
@@ -287,9 +325,27 @@ func NewCluster(opt *NewClusterOptions, clientset simple.Clientset) (*NewCluster
 		return nil, err
 	}
 
-	nodes, err := setupNodes(opt, &cluster, zoneToSubnetMap)
-	if err != nil {
-		return nil, err
+	var nodes []*api.InstanceGroup
+
+	switch opt.InstanceManager {
+	case "karpenter":
+		if opt.DiscoveryStore == "" {
+			return nil, fmt.Errorf("karpenter requires --discovery-store")
+		}
+		cluster.Spec.Karpenter = &api.KarpenterConfig{
+			Enabled: true,
+		}
+		nodes, err = setupKarpenterNodes(opt, &cluster, zoneToSubnetMap)
+		if err != nil {
+			return nil, err
+		}
+	case "cloudgroups":
+		nodes, err = setupNodes(opt, &cluster, zoneToSubnetMap)
+		if err != nil {
+			return nil, err
+		}
+	default:
+		return nil, fmt.Errorf("invalid value %q for --instance-manager", opt.InstanceManager)
 	}
 
 	apiservers, err := setupAPIServers(opt, &cluster, zoneToSubnetMap)
@@ -328,7 +384,7 @@ func NewCluster(opt *NewClusterOptions, clientset simple.Clientset) (*NewCluster
 func setupVPC(opt *NewClusterOptions, cluster *api.Cluster) error {
 	cluster.Spec.NetworkID = opt.NetworkID
 
-	switch api.CloudProviderID(cluster.Spec.CloudProvider) {
+	switch cluster.Spec.GetCloudProvider() {
 	case api.CloudProviderAWS:
 		if cluster.Spec.NetworkID == "" && len(opt.SubnetIDs) > 0 {
 			cloudTags := map[string]string{}
@@ -368,30 +424,12 @@ func setupVPC(opt *NewClusterOptions, cluster *api.Cluster) error {
 			// TODO remove this logging?
 			klog.Infof("VMs will be configured to use specified Service Account: %v", opt.GCEServiceAccount)
 			cluster.Spec.CloudConfig.GCEServiceAccount = opt.GCEServiceAccount
-		} else {
-			klog.Warning("VMs will be configured to use the GCE default compute Service Account! This is an anti-pattern")
-			klog.Warning("Use a pre-created Service Account with the flag: --gce-service-account=account@projectname.iam.gserviceaccount.com")
-			cluster.Spec.CloudConfig.GCEServiceAccount = "default"
 		}
 
 	case api.CloudProviderOpenstack:
 		if cluster.Spec.CloudConfig == nil {
 			cluster.Spec.CloudConfig = &api.CloudConfiguration{}
 		}
-		cluster.Spec.CloudConfig.Openstack = &api.OpenstackConfiguration{
-			Router: &api.OpenstackRouter{
-				ExternalNetwork: fi.String(opt.OpenstackExternalNet),
-			},
-			BlockStorage: &api.OpenstackBlockStorageConfig{
-				Version:  fi.String("v3"),
-				IgnoreAZ: fi.Bool(opt.OpenstackStorageIgnoreAZ),
-			},
-			Monitor: &api.OpenstackMonitor{
-				Delay:      fi.String("1m"),
-				Timeout:    fi.String("30s"),
-				MaxRetries: fi.Int(3),
-			},
-		}
 
 		if cluster.Spec.NetworkID == "" && len(opt.SubnetIDs) > 0 {
 			tags := make(map[string]string)
@@ -409,10 +447,10 @@ func setupVPC(opt *NewClusterOptions, cluster *api.Cluster) error {
 		}
 
 		if opt.OpenstackDNSServers != "" {
-			cluster.Spec.CloudConfig.Openstack.Router.DNSServers = fi.String(opt.OpenstackDNSServers)
+			cluster.Spec.CloudProvider.Openstack.Router.DNSServers = fi.String(opt.OpenstackDNSServers)
 		}
 		if opt.OpenstackExternalSubnet != "" {
-			cluster.Spec.CloudConfig.Openstack.Router.ExternalSubnet = fi.String(opt.OpenstackExternalSubnet)
+			cluster.Spec.CloudProvider.Openstack.Router.ExternalSubnet = fi.String(opt.OpenstackExternalSubnet)
 		}
 	case api.CloudProviderAzure:
 		// TODO(kenji): Find a right place for this.
@@ -421,14 +459,6 @@ func setupVPC(opt *NewClusterOptions, cluster *api.Cluster) error {
 		if cluster.Spec.CloudConfig == nil {
 			cluster.Spec.CloudConfig = &api.CloudConfiguration{}
 		}
-
-		cluster.Spec.CloudConfig.Azure = &api.AzureConfiguration{
-			SubscriptionID:    opt.AzureSubscriptionID,
-			TenantID:          opt.AzureTenantID,
-			ResourceGroupName: opt.AzureResourceGroupName,
-			RouteTableName:    opt.AzureRouteTableName,
-			AdminUser:         opt.AzureAdminUser,
-		}
 	}
 
 	if featureflag.Spotinst.Enabled() {
@@ -456,7 +486,7 @@ func setupZones(opt *NewClusterOptions, cluster *api.Cluster, allZones sets.Stri
 
 	var zoneToSubnetProviderID map[string]string
 
-	switch api.CloudProviderID(cluster.Spec.CloudProvider) {
+	switch cluster.Spec.GetCloudProvider() {
 	case api.CloudProviderGCE:
 		// On GCE, subnets are regional - we create one per region, not per zone
 		for _, zoneName := range allZones.List() {
@@ -513,13 +543,11 @@ func setupZones(opt *NewClusterOptions, cluster *api.Cluster, allZones sets.Stri
 		zoneToSubnetMap[region] = subnet
 		return zoneToSubnetMap, nil
 
-	case api.CloudProviderALI:
-		if len(opt.Zones) > 0 && len(opt.SubnetIDs) > 0 {
-			zoneToSubnetProviderID, err = aliup.ZoneToVSwitchID(cluster.Spec.NetworkID, opt.Zones, opt.SubnetIDs)
-			if err != nil {
-				return nil, err
-			}
+	case api.CloudProviderHetzner:
+		if len(opt.Zones) > 1 {
+			return nil, fmt.Errorf("hetzner cloud provider currently supports only one zone (location)")
 		}
+		// TODO(hakman): Add customizations for Hetzner Cloud
 
 	case api.CloudProviderAzure:
 		// On Azure, subnets are regional - we create one per region, not per zone
@@ -651,7 +679,7 @@ func getOpenstackZoneToSubnetProviderID(spec *api.ClusterSpec, zones []string, s
 }
 
 func setupMasters(opt *NewClusterOptions, cluster *api.Cluster, zoneToSubnetMap map[string]*api.ClusterSubnetSpec) ([]*api.InstanceGroup, error) {
-	cloudProvider := api.CloudProviderID(cluster.Spec.CloudProvider)
+	cloudProvider := cluster.Spec.GetCloudProvider()
 
 	var masters []*api.InstanceGroup
 
@@ -711,6 +739,9 @@ func setupMasters(opt *NewClusterOptions, cluster *api.Cluster, zoneToSubnetMap
 			}
 
 			g.Spec.Subnets = []string{subnet.Name}
+			if opt.IPv6 && opt.Topology == api.TopologyPrivate {
+				g.Spec.Subnets = []string{"dualstack-" + subnet.Name}
+			}
 			if cloudProvider == api.CloudProviderGCE || cloudProvider == api.CloudProviderAzure {
 				g.Spec.Zones = []string{zone}
 			}
@@ -802,7 +833,7 @@ func trimCommonPrefix(names []string) []string {
 }
 
 func setupNodes(opt *NewClusterOptions, cluster *api.Cluster, zoneToSubnetMap map[string]*api.ClusterSubnetSpec) ([]*api.InstanceGroup, error) {
-	cloudProvider := api.CloudProviderID(cluster.Spec.CloudProvider)
+	cloudProvider := cluster.Spec.GetCloudProvider()
 
 	var nodes []*api.InstanceGroup
 
@@ -855,8 +886,22 @@ func setupNodes(opt *NewClusterOptions, cluster *api.Cluster, zoneToSubnetMap ma
 	return nodes, nil
 }
 
+func setupKarpenterNodes(opt *NewClusterOptions, cluster *api.Cluster, zoneToSubnetMap map[string]*api.ClusterSubnetSpec) ([]*api.InstanceGroup, error) {
+	g := &api.InstanceGroup{}
+	g.Spec.Role = api.InstanceGroupRoleNode
+	g.Spec.Manager = api.InstanceManagerKarpenter
+	g.ObjectMeta.Name = "nodes"
+
+	g.Spec.InstanceMetadata = &api.InstanceMetadataOptions{
+		HTTPPutResponseHopLimit: fi.Int64(1),
+		HTTPTokens:              fi.String("required"),
+	}
+
+	return []*api.InstanceGroup{g}, nil
+}
+
 func setupAPIServers(opt *NewClusterOptions, cluster *api.Cluster, zoneToSubnetMap map[string]*api.ClusterSubnetSpec) ([]*api.InstanceGroup, error) {
-	cloudProvider := api.CloudProviderID(cluster.Spec.CloudProvider)
+	cloudProvider := cluster.Spec.GetCloudProvider()
 
 	var nodes []*api.InstanceGroup
 
@@ -921,7 +966,7 @@ func setupNetworking(opt *NewClusterOptions, cluster *api.Cluster) error {
 	case "weave":
 		cluster.Spec.Networking.Weave = &api.WeaveNetworkingSpec{}
 
-		if cluster.Spec.CloudProvider == "aws" {
+		if cluster.Spec.GetCloudProvider() == api.CloudProviderAWS {
 			// AWS supports "jumbo frames" of 9001 bytes and weave adds up to 87 bytes overhead
 			// sets the default to the largest number that leaves enough overhead and is divisible by 4
 			jumboFrameMTUSize := int32(8912)
@@ -1000,7 +1045,7 @@ func setupTopology(opt *NewClusterOptions, cluster *api.Cluster, allZones sets.S
 		var zoneToSubnetProviderID map[string]string
 		var err error
 		if len(opt.Zones) > 0 && len(opt.UtilitySubnetIDs) > 0 {
-			switch api.CloudProviderID(cluster.Spec.CloudProvider) {
+			switch cluster.Spec.GetCloudProvider() {
 			case api.CloudProviderAWS:
 				zoneToSubnetProviderID, err = getAWSZoneToSubnetProviderID(cluster.Spec.NetworkID, opt.Zones[0][:len(opt.Zones[0])-1], opt.UtilitySubnetIDs)
 				if err != nil {
@@ -1016,8 +1061,29 @@ func setupTopology(opt *NewClusterOptions, cluster *api.Cluster, allZones sets.S
 			}
 		}
 
+		if opt.IPv6 {
+			var dualStackSubnets []api.ClusterSubnetSpec
+
+			for _, s := range cluster.Spec.Subnets {
+				if s.Type != api.SubnetTypePrivate {
+					continue
+				}
+				subnet := api.ClusterSubnetSpec{
+					Name:   "dualstack-" + s.Name,
+					Zone:   s.Zone,
+					Type:   api.SubnetTypeDualStack,
+					Region: s.Region,
+				}
+				if subnetID, ok := zoneToSubnetProviderID[s.Zone]; ok {
+					subnet.ProviderID = subnetID
+				}
+				dualStackSubnets = append(dualStackSubnets, subnet)
+			}
+			cluster.Spec.Subnets = append(cluster.Spec.Subnets, dualStackSubnets...)
+		}
+
 		addUtilitySubnets := true
-		switch api.CloudProviderID(cluster.Spec.CloudProvider) {
+		switch cluster.Spec.GetCloudProvider() {
 		case api.CloudProviderGCE:
 			// GCE does not need utility subnets
 			addUtilitySubnets = false
@@ -1027,7 +1093,7 @@ func setupTopology(opt *NewClusterOptions, cluster *api.Cluster, allZones sets.S
 			var utilitySubnets []api.ClusterSubnetSpec
 
 			for _, s := range cluster.Spec.Subnets {
-				if s.Type == api.SubnetTypeUtility {
+				if s.Type != api.SubnetTypePrivate {
 					continue
 				}
 				subnet := api.ClusterSubnetSpec{
@@ -1055,7 +1121,7 @@ func setupTopology(opt *NewClusterOptions, cluster *api.Cluster, allZones sets.S
 					PublicName: "bastion." + cluster.Name,
 				}
 			}
-			if api.CloudProviderID(cluster.Spec.CloudProvider) == api.CloudProviderGCE {
+			if cluster.Spec.GetCloudProvider() == api.CloudProviderGCE {
 				bastionGroup.Spec.Zones = allZones.List()
 			}
 
@@ -1075,7 +1141,7 @@ func setupTopology(opt *NewClusterOptions, cluster *api.Cluster, allZones sets.S
 	if opt.IPv6 {
 		cluster.Spec.NonMasqueradeCIDR = "::/0"
 		cluster.Spec.ExternalCloudControllerManager = &api.CloudControllerManagerConfig{}
-		if api.CloudProviderID(cluster.Spec.CloudProvider) == api.CloudProviderAWS {
+		if cluster.Spec.GetCloudProvider() == api.CloudProviderAWS {
 			for i := range cluster.Spec.Subnets {
 				cluster.Spec.Subnets[i].IPv6CIDR = fmt.Sprintf("/64#%x", i)
 			}
@@ -1099,11 +1165,11 @@ func setupTopology(opt *NewClusterOptions, cluster *api.Cluster, allZones sets.S
 
 func setupAPI(opt *NewClusterOptions, cluster *api.Cluster) error {
 	// Populate the API access, so that it can be discoverable
-	klog.Infof(" Cloud Provider ID = %s", api.CloudProviderID(cluster.Spec.CloudProvider))
+	klog.Infof(" Cloud Provider ID = %s", cluster.Spec.GetCloudProvider())
 	cluster.Spec.API = &api.AccessSpec{}
-	if api.CloudProviderID(cluster.Spec.CloudProvider) == api.CloudProviderOpenstack {
+	if cluster.Spec.GetCloudProvider() == api.CloudProviderOpenstack {
 		initializeOpenstackAPI(opt, cluster)
-	} else if api.CloudProviderID(cluster.Spec.CloudProvider) == api.CloudProviderAzure {
+	} else if cluster.Spec.GetCloudProvider() == api.CloudProviderAzure {
 		// Do nothing to disable the use of loadbalancer for the k8s API server.
 		// TODO(kenji): Remove this condition once we support the loadbalancer
 		// in pkg/model/azuremodel/api_loadbalancer.go.
@@ -1144,7 +1210,7 @@ func setupAPI(opt *NewClusterOptions, cluster *api.Cluster) error {
 		cluster.Spec.API.LoadBalancer.SSLCertificate = opt.APISSLCertificate
 	}
 
-	if cluster.Spec.API.LoadBalancer != nil && cluster.Spec.API.LoadBalancer.Class == "" && api.CloudProviderID(cluster.Spec.CloudProvider) == api.CloudProviderAWS {
+	if cluster.Spec.API.LoadBalancer != nil && cluster.Spec.API.LoadBalancer.Class == "" && cluster.Spec.GetCloudProvider() == api.CloudProviderAWS {
 		switch opt.APILoadBalancerClass {
 		case "", "classic":
 			cluster.Spec.API.LoadBalancer.Class = api.LoadBalancerClassClassic
@@ -1170,15 +1236,19 @@ func initializeOpenstackAPI(opt *NewClusterOptions, cluster *api.Cluster) {
 			}
 		}
 
-		cluster.Spec.CloudConfig.Openstack.Loadbalancer = &api.OpenstackLoadbalancerConfig{
+		LbMethod := "ROUND_ROBIN"
+		if provider == "ovn" {
+			LbMethod = "SOURCE_IP_PORT"
+		}
+		cluster.Spec.CloudProvider.Openstack.Loadbalancer = &api.OpenstackLoadbalancerConfig{
 			FloatingNetwork: fi.String(opt.OpenstackExternalNet),
-			Method:          fi.String("ROUND_ROBIN"),
+			Method:          fi.String(LbMethod),
 			Provider:        fi.String(provider),
 			UseOctavia:      fi.Bool(opt.OpenstackLBOctavia),
 		}
 
 		if opt.OpenstackLBSubnet != "" {
-			cluster.Spec.CloudConfig.Openstack.Loadbalancer.FloatingSubnet = fi.String(opt.OpenstackLBSubnet)
+			cluster.Spec.CloudProvider.Openstack.Loadbalancer.FloatingSubnet = fi.String(opt.OpenstackLBSubnet)
 		}
 	}
 }
